@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2025 Colin Ian King
+ * Copyright (C) 2022-2026 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -61,6 +61,16 @@
 #define	shim_strdup(str)		strdup((str))
 #endif
 
+#if defined(HAVE_BUILTIN_COMPLEX)
+#define shim_cmplxf(r, i)		__builtin_complex((float)(r), (float)(i))
+#define shim_cmplx(r, i)		__builtin_complex((double)(r), (double)(i))
+#define shim_cmplxl(r, i)		__builtin_complex((long double)(r), (long double)(i))
+#else
+#define shim_cmplxf(r, i)		((float)(r) + (I * (float)(i)))
+#define shim_cmplx(r, i)		((double)(r) + (I * (double)(i)))
+#define shim_cmplxl(r, i)		((long double)(r) + (I * (long double)(i)))
+#endif
+
 #if defined(HAVE_BUILTIN_CABS)
 #define shim_cabs(x)		__builtin_cabs((x))
 #else
@@ -109,7 +119,7 @@
 #if defined(HAVE_CPOWF)
 #define shim_cpowf(x, z)	cpowf((x), (z))
 #else
-#define shim_cpowf(x, z)	(shim_cexpf(z) * shim_clogf((x))))
+#define shim_cpowf(x, z)	(shim_cexpf(z) * shim_clogf((x)))
 #endif
 #endif
 
@@ -119,7 +129,7 @@
 #if defined(HAVE_CPOWL)
 #define shim_cpowl(x, z)	cpowl((x), (z))
 #else
-#define shim_cpowl(x, z)	(shim_cexpl((z) * shim_clogl((l))))
+#define shim_cpowl(x, z)	(shim_cexpl((z) * shim_clogl((x))))
 #endif
 #endif
 
@@ -380,7 +390,7 @@
 #if defined(HAVE_BUILTIN_CBRTL)
 #define shim_cbrtl(x)		__builtin_cbrtl((x))
 #else
-#if defined(HAVE_SQRTL)
+#if defined(HAVE_CSQRTL)
 #define shim_cbrtl(x)		cbrtl((x))
 #else
 #define shim_cbrtl(x)		((long double)shim_cbrt((double)(x)))
@@ -465,7 +475,7 @@
 #define shim_ccosh(x)		ccosh((x))
 #endif
 
-#if defined(HAVE_BUILTIN_COSHL)
+#if defined(HAVE_BUILTIN_CCOSHL)
 #define shim_ccoshl(x)		__builtin_ccoshl((x))
 #else
 #if defined(HAVE_CCOSHL)
