@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -84,7 +84,7 @@ void stress_check_range(
 }
 
 /*
- *  stress_check_range()
+ *  stress_check_range_bytes()
  *	Sanity check val against a lo - hi range
  */
 void stress_check_range_bytes(
@@ -449,7 +449,7 @@ static const stress_scale_t size_scales[] = {
 uint64_t stress_get_uint64_byte_scale(const char *const str)
 {
 	int ch, i;
-	const int len = strlen(str);
+	const size_t len = strlen(str);
 
 	if (len < 1) {
 		(void)fprintf(stderr, "Illegal empty specifier\n");
@@ -464,11 +464,11 @@ uint64_t stress_get_uint64_byte_scale(const char *const str)
 	}
 
 illegal:
-	(void)fprintf(stderr, "Illegal specifier '%s', allower specifiers: ", str);
+	(void)fprintf(stderr, "Illegal specifier '%s', allowed specifiers: ", str);
 err:
 
 	for (i = 1; size_scales[i].ch; i++) {
-		fprintf(stderr, "%s%c", ((i == 0) ? "" : ", "), size_scales[i].ch);
+		fprintf(stderr, "%s%c", ((i == 1) ? "" : ", "), size_scales[i].ch);
 	}
 	fprintf(stderr, "\n");
 	longjmp(g_error_env, 1);
@@ -556,7 +556,7 @@ uint64_t stress_get_uint64_percent(
 		percent = ((double)max * (double)val) / (100.0 * (double)instances);
 		if (percentage)
 			*percentage = true;
-		if ((uint64_t)percent > UINT64_MAX) {
+		if (percent > (double)UINT64_MAX) {
 			(void)fprintf(stderr, "Invalid too large percentage %s\n", str);
 			longjmp(g_error_env, 1);
 			stress_no_return();
@@ -817,7 +817,7 @@ int stress_parse_opt(const char *stressor_name, const char *opt_arg, const stres
  *	method handler for methods that are unimplemented
  *	(e.g. stressor not supported)
  */
-const char * PURE stress_unimplemented_method(const size_t i)
+const char * CONST stress_unimplemented_method(const size_t i)
 {
 	(void)i;
 
