@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -99,14 +99,14 @@ typedef struct stress_crypto_info {
 	stress_crypto_source_t	source;
 	char 	*type;
 	char 	*name;
-	int 	block_size;
-	int	max_key_size;
-	int	max_auth_size;
-	int	iv_size;
-	int	digest_size;
-	bool	internal;		/* true if accessible to userspace */
-	bool	ignore;
-	bool	selftest;		/* true if passed */
+	int8_t	block_size;
+	int8_t	max_key_size;
+	int8_t	max_auth_size;
+	int8_t	iv_size;
+	int8_t	digest_size;
+	uint8_t	internal:1;		/* true if accessible to userspace */
+	uint8_t	ignore:1;
+	uint8_t	selftest:1;		/* true if passed */
 	stress_metrics_t metrics;	/* performance metrics */
 	struct stress_crypto_info *next;
 } stress_crypto_info_t;
@@ -140,6 +140,7 @@ static void MLOCKED_TEXT stress_af_alg_alarm_handler(int signum)
 	if (UNLIKELY(do_jmp && count++ > 5)) {
 		do_jmp = false;
 		siglongjmp(jmpbuf, 1);
+		stress_no_return();
 	}
 }
 
@@ -149,13 +150,13 @@ static void MLOCKED_TEXT stress_af_alg_alarm_handler(int signum)
  */
 static stress_crypto_type_t name_to_type(const char *buffer)
 {
-	const char *end, *ptr = strchr(buffer, ':');
+	const char *end = buffer + strlen(buffer);
+	const char *ptr = strchr(buffer, ':');
 	size_t i;
 
-	if (!ptr)
+	if (UNLIKELY(!ptr))
 		return CRYPTO_UNKNOWN;
 
-	end = ptr + strlen(buffer);
 	ptr += 2;
 	if (ptr >= end)
 		return CRYPTO_UNKNOWN;
@@ -193,7 +194,7 @@ static void stress_af_alg_ignore(
 	stress_crypto_info_t *info,
 	const char *systemcall)
 {
-	if ((args->instance == 0) && (!info->ignore)) {
+	if ((stress_instance_zero(args)) && (!info->ignore)) {
 		pr_dbg_skip("%s: %s using %s (%s), failed with EINVAL, skipping this crypto engine\n",
 			args->name, systemcall, info->name, info->type);
 	}
@@ -256,6 +257,8 @@ retry_bind:
 				goto retry_bind;
 			rc = EXIT_NO_RESOURCE;
 			goto err;
+		default:
+			break;
 		}
 		pr_fail("%s: %s (%s): bind failed, errno=%d (%s)\n",
 			args->name, info->name, info->type, errno, strerror(errno));
@@ -520,7 +523,7 @@ retry_bind:
 		cmsg->cmsg_level = SOL_ALG;
 		cmsg->cmsg_type = ALG_SET_IV;
 		cmsg->cmsg_len = CMSG_LEN(4) + CMSG_LEN(iv_size);
-		iv = (void *)CMSG_DATA(cmsg);
+		iv = (struct af_alg_iv *)shim_assume_aligned(CMSG_DATA(cmsg), 1);
 		iv->ivlen = (uint32_t)iv_size;
 
 		stress_rndbuf((char *)iv->iv, (size_t)iv_size);
@@ -587,7 +590,7 @@ retry_bind:
 		cmsg->cmsg_level = SOL_ALG;
 		cmsg->cmsg_type = ALG_SET_IV;
 		cmsg->cmsg_len = CMSG_LEN(4) + CMSG_LEN(iv_size);
-		iv = (void *)CMSG_DATA(cmsg);
+		iv = (struct af_alg_iv *)shim_assume_aligned(CMSG_DATA(cmsg), 1);
 		iv->ivlen = (uint32_t)iv_size;
 
 		iov.iov_base = output;
@@ -775,7 +778,7 @@ static void stress_af_alg_count_crypto(size_t *count, size_t *internal)
  *  stress_af_alg_cmp_crypto()
  *	qsort comparison on type then name
  */
-static int PURE stress_af_alg_cmp_crypto(const void *p1, const void *p2)
+static int CONST stress_af_alg_cmp_crypto(const void *p1, const void *p2)
 {
 	int n;
 	const stress_crypto_info_t * const *ci1 = (const stress_crypto_info_t * const *)p1;
@@ -880,7 +883,7 @@ static int stress_af_alg(stress_args_t *args)
 
 	(void)stress_get_setting("af-alg-dump", &af_alg_dump);
 
-	if (af_alg_dump && (args->instance == 0)) {
+	if (af_alg_dump && stress_instance_zero(args)) {
 		pr_inf("%s: dumping cryptographic algorithms found in /proc/crypto to stdout\n",
 			args->name);
 		stress_af_alg_sort_crypto();
@@ -891,14 +894,14 @@ static int stress_af_alg(stress_args_t *args)
 	stress_af_alg_sort_crypto();
 	stress_af_alg_count_crypto(&count, &internal);
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_block_begin();
-		pr_inf("%s: %zd cryptographic algorithms found in /proc/crypto\n",
+		pr_inf("%s: %zu cryptographic algorithms found in /proc/crypto\n",
 			args->name, proc_count);
-		pr_inf("%s: %zd cryptographic algorithms in total (with defconfigs)\n",
+		pr_inf("%s: %zu cryptographic algorithms in total (with defconfigs)\n",
 			args->name, count);
 		if (internal)
-			pr_inf("%s: %zd cryptographic algorithms are internal and may be unused\n",
+			pr_inf("%s: %zu cryptographic algorithms are internal and may be unused\n",
 				args->name, internal);
 		pr_block_end();
 	}
@@ -972,6 +975,7 @@ static int stress_af_alg(stress_args_t *args)
 					goto deinit;
 				break;
 			case CRYPTO_UNKNOWN:
+				break;
 			default:
 				break;
 			}
@@ -988,8 +992,7 @@ deinit:
 			const double rate = info->metrics.count / info->metrics.duration;
 			char str[64];
 
-			(void)snprintf(str, sizeof(str), "%s (%s) ops/sec", info->name, info->type),
-
+			(void)snprintf(str, sizeof(str), "%s (%s) ops/sec", info->name, info->type);
 			stress_metrics_set(args, idx, str, rate, STRESS_METRIC_HARMONIC_MEAN);
 			idx++;
 		}
@@ -1004,7 +1007,7 @@ deinit:
  *  dup_field()
  * 	duplicate a text string field
  */
-static char *dup_field(const char *buffer)
+static char *dup_field(char *buffer)
 {
 	const char *ptr = strchr(buffer, ':');
 	char *eol = strchr(buffer, '\n');
@@ -1014,14 +1017,14 @@ static char *dup_field(const char *buffer)
 	if (eol)
 		*eol = '\0';
 
-	return strdup(ptr + 2);
+	return shim_strdup(ptr + 2);
 }
 
 /*
  *  int_field()
  *	parse an integer from a numeric field
  */
-static int PURE int_field(const char *buffer)
+static int CONST int_field(const char *buffer)
 {
 	const char *ptr = strchr(buffer, ':');
 
@@ -1035,7 +1038,7 @@ static int PURE int_field(const char *buffer)
  *	parse a boolean from a string field
  *	error/default is false.
  */
-static bool PURE bool_field(const char *buffer)
+static bool CONST bool_field(const char *buffer)
 {
 	const char *ptr = strchr(buffer, ':');
 
@@ -1170,15 +1173,15 @@ static void stress_af_alg_init(const uint32_t instances)
 			info.type = dup_field(buffer);
 		}
 		else if (!strncmp(buffer, "blocksize", 9))
-			info.block_size = int_field(buffer);
+			info.block_size = (int8_t)int_field(buffer);
 		else if (!strncmp(buffer, "max keysize", 11))
-			info.max_key_size = int_field(buffer);
+			info.max_key_size = (int8_t)int_field(buffer);
 		else if (!strncmp(buffer, "maxauthsize", 11))
-			info.max_auth_size = int_field(buffer);
+			info.max_auth_size = (int8_t)int_field(buffer);
 		else if (!strncmp(buffer, "ivsize", 6))
-			info.iv_size = int_field(buffer);
+			info.iv_size = (int8_t)int_field(buffer);
 		else if (!strncmp(buffer, "digestsize", 10))
-			info.digest_size = int_field(buffer);
+			info.digest_size = (int8_t)int_field(buffer);
 		else if (!strncmp(buffer, "internal", 8))
 			info.internal = bool_field(buffer);
 		else if (!strncmp(buffer, "selftest", 8))
@@ -1224,7 +1227,7 @@ const stressor_info_t stress_af_alg_info = {
 	.stressor = stress_af_alg,
 	.init = stress_af_alg_init,
 	.deinit = stress_af_alg_deinit,
-	.class = CLASS_CPU | CLASS_OS,
+	.classifier = CLASS_CPU | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -1233,7 +1236,7 @@ const stressor_info_t stress_af_alg_info = {
 #else
 const stressor_info_t stress_af_alg_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_OS,
+	.classifier = CLASS_CPU | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
