@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -57,7 +57,7 @@ static size_t stress_minsigstksz;
 
 STRESS_PRAGMA_PUSH
 STRESS_PRAGMA_WARN_OFF
-static void stress_bad_altstack_force_fault(uint8_t *stack_start)
+static void stress_bad_altstack_force_fault(void *stack_start)
 {
 	volatile uint8_t *vol_stack = (volatile uint8_t *)stack_start;
 	/* trigger segfault on stack */
@@ -237,7 +237,10 @@ retry:
 				stress_bad_altstack_force_fault(stack);
 			goto retry;
 #else
-		case 1..4:
+		case 1:
+		case 2:
+		case 3:
+		case 4:
 			goto retry;
 #endif
 		case 5:
@@ -301,7 +304,7 @@ retry:
 	}
 	/* No luck, well that's unexpected.. */
 	if (UNLIKELY(!stress_continue(args)))
-		pr_fail("%s: child process with illegal stack unexpectedly worked, %d\n",
+		pr_fail("%s: child process with illegal stack unexpectedly worked, %" PRIu32 "\n",
 			args->name, rnd);
 	return EXIT_FAILURE;
 }
@@ -416,7 +419,7 @@ again:
 						stress_log_system_mem_info();
 						pr_dbg("%s: assuming killed by OOM "
 							"killer, bailing out "
-							"(instance %d)\n",
+							"(instance %" PRIu32 ")\n",
 							args->name, args->instance);
 						rc = EXIT_SUCCESS;
 						goto finish;
@@ -424,7 +427,7 @@ again:
 						stress_log_system_mem_info();
 						pr_dbg("%s: assuming killed by OOM "
 							"killer, restarting again "
-							"(instance %d)\n",
+							"(instance %" PRIu32 ")\n",
 							args->name, args->instance);
 						goto again;
 					}
@@ -440,6 +443,7 @@ again:
 			}
 		} else {
 			stress_set_proc_state(args->name, STRESS_STATE_RUN);
+			stress_set_make_it_fail();
 			_exit(stress_bad_altstack_child(args));
 		}
 	} while (stress_continue(args));
