@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -175,15 +175,15 @@ static int OPTIMIZE3 stress_bsearch(stress_args_t *args)
 		for (ptr = data, i = 0; i < n; i++, ptr++) {
 			int32_t *result;
 
-			result = bsearch_func(ptr, data, n, sizeof(*ptr), stress_sort_cmp_fwd_int32);
+			result = (int32_t *)bsearch_func(ptr, data, n, sizeof(*ptr), stress_sort_cmp_fwd_int32);
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				if (result == NULL)
 					pr_fail("%s: element %zu could not be found\n",
 						args->name, i);
 				else if (*result != *ptr) {
 					pr_fail("%s: element %zu "
-						"found %" PRIu32
-						", expecting %" PRIu32 "\n",
+						"found %" PRId32
+						", expecting %" PRId32 "\n",
 						args->name, i, *result, *ptr);
 					rc = EXIT_FAILURE;
 					break;
@@ -201,8 +201,9 @@ static int OPTIMIZE3 stress_bsearch(stress_args_t *args)
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "bsearch comparisons per sec",
 		rate, STRESS_METRIC_HARMONIC_MEAN);
+	rate = (sorted > 0.0) ? count / sorted : 0.0;
 	stress_metrics_set(args, 1, "bsearch comparisons per item",
-		count / sorted, STRESS_METRIC_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	pr_dbg("%s: %.2f bsearch comparisons per sec\n", args->name, rate);
 
@@ -212,7 +213,7 @@ static int OPTIMIZE3 stress_bsearch(stress_args_t *args)
 
 static const stress_opt_t opts[] = {
 	{ OPT_bsearch_size,   "bsearch-size",   TYPE_ID_UINT64,        MIN_BSEARCH_SIZE, MAX_BSEARCH_SIZE, NULL },
-	{ OPT_bsearch_method, "bsearch-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_bsearch_method },
+	{ OPT_bsearch_method, "bsearch-method", TYPE_ID_SIZE_T_METHOD, 0, 0, (void *)stress_bsearch_method },
 	END_OPT,
 };
 
