@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2024-2025 Colin Ian King
+ * Copyright (C) 2024-2026 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@
 #include "core-numa.h"
 #include "core-put.h"
 
+#include <math.h>
 #include <sched.h>
 
 #define N_FUNCS (SIZEOF_ARRAY(stress_cachehammer_funcs))
@@ -98,10 +99,10 @@ static void stress_cachehammer_init(const uint32_t instances)
 
 	(void)instances;
 
-	(void)memset(cachehammer_filename, 0, sizeof(cachehammer_filename));
-	(void)memset(cachehammer_path, 0, sizeof(cachehammer_path));
+	(void)shim_memset(cachehammer_filename, 0, sizeof(cachehammer_filename));
+	(void)shim_memset(cachehammer_path, 0, sizeof(cachehammer_path));
 
-	page = calloc(page_size, sizeof(*page));
+	page = (uint8_t *)calloc(page_size, sizeof(*page));
 	if (!page)
 		goto err_nullstr;
 	if (stress_temp_dir(cachehammer_path, sizeof(cachehammer_path),
@@ -241,14 +242,14 @@ static void OPTIMIZE3 hammer_write(
 		if (UNLIKELY(val != pattern)) {
 			pr_fail("%s: write: read back of stored value at address "
 				"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-				args->name, vptr, pattern, val);
+				args->name, (volatile void *)vptr, pattern, val);
 		}
 		vptr = (volatile uint64_t *)addr2;
 		val = *vptr;
 		if (UNLIKELY(val != pattern)) {
 			pr_fail("%s: write: read back of stored value at address "
 				"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-				args->name, vptr, pattern, val);
+				args->name, (volatile void *)vptr, pattern, val);
 		}
 	}
 }
@@ -300,7 +301,7 @@ static void OPTIMIZE3 hammer_write64(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write64: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -341,14 +342,14 @@ static void OPTIMIZE3 hammer_readwrite(
 		if (UNLIKELY(val != pattern)) {
 			pr_fail("%s: readwrite: read back of stored value at address "
 				"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-				args->name, vptr, pattern, val);
+				args->name, (volatile void *)vptr, pattern, val);
 		}
 		vptr = (volatile uint64_t *)addr2;
 		val = *vptr;
 		if (UNLIKELY(val != pattern)) {
 			pr_fail("%s: readwrite: read back of stored value at address "
 				"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-				args->name, vptr, pattern, val);
+				args->name, (volatile void *)vptr, pattern, val);
 		}
 	}
 }
@@ -428,14 +429,14 @@ static void OPTIMIZE3 hammer_writeread(
 		if (UNLIKELY(val != pattern)) {
 			pr_fail("%s: writeread: read back of stored value at address "
 				"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-				args->name, vptr, pattern, val);
+				args->name, (volatile void *)vptr, pattern, val);
 		}
 		vptr = (volatile uint64_t *)addr2;
 		val = *vptr;
 		if (UNLIKELY(val != pattern)) {
 			pr_fail("%s: writeread: read back of stored value at address "
 				"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-				args->name, vptr, pattern, val);
+				args->name, (volatile void *)vptr, pattern, val);
 		}
 	}
 }
@@ -487,7 +488,7 @@ static void OPTIMIZE3 hammer_writeread64(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: writeread64: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr1, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -602,14 +603,14 @@ static void OPTIMIZE3 hammer_ppc64_dcbst(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -658,14 +659,14 @@ static void OPTIMIZE3 hammer_ppc_dcbst(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -792,14 +793,14 @@ static void OPTIMIZE3 hammer_ppc64_dcbtst(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbtst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbtst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -848,14 +849,14 @@ static void OPTIMIZE3 hammer_ppc_dcbtst(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbtst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: dcbtst: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -1272,14 +1273,14 @@ static void OPTIMIZE3 hammer_write_clflush(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write-clflush: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write-clflush: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -1326,14 +1327,14 @@ static void OPTIMIZE3 hammer_write_clflushopt(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write-clflushopt: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write-clflushopt: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -1407,14 +1408,14 @@ static void OPTIMIZE3 hammer_clwb(
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write-clwb: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 			vptr = (volatile uint64_t *)addr2;
 			val = *vptr;
 			if (UNLIKELY(val != pattern)) {
 				pr_fail("%s: write-clwb: read back of stored value at address "
 					"%p not %" PRIx64 ", got %" PRIx64 " instead\n",
-					args->name, vptr, pattern, val);
+					args->name, (volatile void *)vptr, pattern, val);
 			}
 		}
 	}
@@ -1576,6 +1577,7 @@ static const stress_cachehammer_func_t stress_cachehammer_funcs[] = {
 	{ "write-read64", hammer_valid,			hammer_writeread64 },
 };
 
+static stress_metrics_t cachehammer_metrics[N_FUNCS];
 static bool valid[N_FUNCS];
 static bool trapped[N_FUNCS];
 static size_t func_index;
@@ -1658,12 +1660,14 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 	const size_t buffer_size = (size_t)g_shared->mem_cache.size;
 	const size_t local_buffer_size = buffer_size * 4;
 	const uint32_t mask = ~0x3f;
-	const uint32_t page_mask = (page_size - 1) & ~0x3f;
-	size_t i;
+	const uint32_t page_mask = (uint32_t)((page_size - 1) & ~0x3f);
+	size_t i, j;
 	NOCLOBBER size_t tries = 0;
 	char buf[1024];
 	NOCLOBBER bool cachehammer_numa = false;
 	static int numa_count[5];
+	double mantissa;
+	uint64_t exponent;
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	NOCLOBBER stress_numa_mask_t *numa_mask;
 	NOCLOBBER stress_numa_mask_t *numa_nodes;
@@ -1681,9 +1685,11 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 	func_index = 0;
-	for (i = 0; i < SIZEOF_ARRAY(stress_cachehammer_funcs); i++) {
+	for (i = 0; i < N_FUNCS; i++) {
 		valid[i] = stress_cachehammer_funcs[i].valid();
 		trapped[i] = false;
+		cachehammer_metrics[i].duration = 0.0;
+		cachehammer_metrics[i].count = 0.0;
 	}
 
 	if (stress_instance_zero(args)) {
@@ -1761,7 +1767,8 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 
 	if (cachehammer_numa) {
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-		stress_numa_mask_and_node_alloc(args, &numa_nodes, &numa_mask, "--cachehammer-numa", &cachehammer_numa);
+		stress_numa_mask_and_node_alloc(args, &numa_nodes, &numa_mask,
+						"--cachehammer-numa", &cachehammer_numa);
 #else
 		if (stress_instance_zero(args))
 			pr_inf("%s: --cachehammer numa selected but not supported by this system, disabling option\n",
@@ -1775,22 +1782,11 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	if (cachehammer_numa) {
-#if defined(HAVE_LINUX_MEMPOLICY_H)
-		stress_numa_mask_and_node_alloc(args, &numa_nodes, &numa_mask,
-						"--cachehammer-numa", &cachehammer_numa);
-#else
-		if (stress_instance_zero(args))
-			pr_inf("%s: --cachehammer-numa selected but not supported by this system, disabling option\n",
-				args->name);
-		cachehammer_numa = false;
-#endif
-	}
-
 	(void)sigsetjmp(jmp_env, 1);
 	func_index = stress_mwc32modn((uint32_t)N_FUNCS);
 	while (stress_continue(args)) {
 		if (valid[func_index] && !trapped[func_index]) {
+			double t_start;
 			const uint16_t rnd16 = stress_mwc16();
 			const size_t loops = 8 + ((rnd16 >> 1) & 0x3f);
 			const uint8_t which = (rnd16 == 0x0008) ? 4 : rnd16 & 3;
@@ -1799,6 +1795,7 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 			uint32_t offset;
 			register uint8_t *addr1, *addr2;
 
+			t_start = stress_time_now();
 			switch (which) {
 			case 0:
 				(*file_page)++;
@@ -1884,6 +1881,8 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 				hammer(args, addr1, addr2, true, false);
 				break;
 			}
+			cachehammer_metrics[func_index].duration += stress_time_now() - t_start;
+			cachehammer_metrics[func_index].count += 1.0;
 			tries = 0;
 			stress_bogo_inc(args);
 			func_index = stress_mwc32modn((uint32_t)N_FUNCS);
@@ -1911,6 +1910,35 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	mantissa = 1.0;
+	exponent = 0;
+
+	for (i = 0, j = 0; i < N_FUNCS; i++) {
+		if (cachehammer_metrics[i].duration > 0.0) {
+			char msg[64];
+			int e;
+			const double rate = cachehammer_metrics[i].count / cachehammer_metrics[i].duration;
+			const double f = frexp(rate, &e);
+
+			mantissa *= f;
+			exponent += e;
+
+			(void)snprintf(msg, sizeof(msg), "%s cache bogo-ops/sec",
+				 stress_cachehammer_funcs[i].name);
+			stress_metrics_set(args, j, msg, rate, STRESS_METRIC_HARMONIC_MEAN);
+			j++;
+		}
+	}
+
+	if (j > 0) {
+		double inverse_n = 1.0 / (double)j;
+		double geomean = pow(mantissa, inverse_n) *
+				 pow(2.0, (double)exponent * inverse_n);
+
+		pr_dbg("%s: %.2f cachehammer ops per second (geometric mean of per stressor bogo-op rates)\n",
+			args->name, geomean);
+	}
+
 #if defined(HAVE_LINUX_MEMPOLICY_H)
         if (numa_mask)
                 stress_numa_mask_free(numa_mask);
