@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,16 +20,18 @@
 #include "stress-ng.h"
 #include "core-put.h"
 
-static sigjmp_buf jmp_env;
-static volatile bool do_jmp = true;
-static volatile int die_signum = -1;
-
 static const stress_help_t help[] = {
 	{ NULL,	"fault N",	"start N workers producing page faults" },
 	{ NULL,	"fault-ops N",	"stop after N page fault bogo operations" },
 	{ NULL,	NULL,		NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+static sigjmp_buf jmp_env;
+static volatile bool do_jmp = true;
+static volatile int die_signum = -1;
+
 /*
  *  stress_segvhandler()
  *	SEGV handler
@@ -38,8 +40,10 @@ static void MLOCKED_TEXT stress_segvhandler(int signum)
 {
 	die_signum = signum;
 
-	if (do_jmp)
+	if (do_jmp) {
 		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+		stress_no_return();
+	}
 }
 
 /*
@@ -96,7 +100,7 @@ static int stress_fault(stress_args_t *args)
 #endif
 	i = 0;
 	do {
-		int fd;
+		int fd = -1;
 		uint8_t *ptr;
 		double t;
 
@@ -126,8 +130,9 @@ static int stress_fault(stress_args_t *args)
 				continue;	/* Try again */
 			}
 			(void)close(fd);
+			/* posix_fallocate errno is in ret */
 			pr_fail("%s: posix_fallocate failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+				args->name, ret, strerror(ret));
 			rc = EXIT_FAILURE;
 			break;
 		}
@@ -177,8 +182,9 @@ redo:
 				(void)close(fd);
 				goto next;
 			}
-			pr_err("%s: mmap failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_err("%s: mmap of 1 byte failed%s, errno=%d (%s)\n",
+				args->name, stress_get_memfree_str(),
+				errno, strerror(errno));
 			(void)close(fd);
 			break;
 
@@ -209,7 +215,7 @@ redo:
 			count += 1.0;
 		}
 #endif
-		if (stress_munmap_retry_enomem((void *)ptr, page_size) < 0) {
+		if (stress_munmap_force((void *)ptr, page_size) < 0) {
 			pr_err("%s: munmap failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
@@ -241,7 +247,7 @@ next:
 						count += 1.0;
 					}
 #endif
-					(void)stress_munmap_retry_enomem((void *)ptr, page_size);
+					(void)stress_munmap_force((void *)ptr, page_size);
 				}
 			}
 		}
@@ -270,7 +276,7 @@ next:
     defined(RUSAGE_SELF) &&		\
     defined(HAVE_RUSAGE_RU_MINFLT)
 	if (!shim_getrusage(RUSAGE_SELF, &usage)) {
-		pr_dbg("%s: page faults: minor: %lu, major: %lu\n",
+		pr_dbg("%s: page faults: minor: %ld, major: %ld\n",
 			args->name, usage.ru_minflt, usage.ru_majflt);
 	}
 	dt = t2 - t1;
@@ -291,6 +297,17 @@ next:
 
 const stressor_info_t stress_fault_info = {
 	.stressor = stress_fault,
-	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_fault_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
