@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -55,8 +55,8 @@ static const stress_help_t help[] = {
      defined(F_SETLKW) && defined(F_WRLCK) && \
      defined(F_UNLCK)) ||	\
     (defined(F_OFD_GETLK) && defined(F_OFD_SETLK) && \
-     defined(F_OFD_SETLKW) && defined(F_WRLCK) && defined(F_UNLCK)) | \
-    (defined(F_GET_FILE_RW_HINT) && defined(F_SET_FILE_RW_HINT)) | \
+     defined(F_OFD_SETLKW) && defined(F_WRLCK) && defined(F_UNLCK)) || \
+    (defined(F_GET_FILE_RW_HINT) && defined(F_SET_FILE_RW_HINT)) || \
     (defined(F_GET_RW_HINT) && defined(F_SET_RW_HINT))
 
 static size_t setfl_flag_count;
@@ -133,7 +133,7 @@ static void do_fcntl(
 	{
 		int ret;
 
-		ret = fcntl(fd, F_DUPFD, F_DUPFD_CLOEXEC);
+		ret = fcntl(fd, F_DUPFD_CLOEXEC, 0);
 		check_return(args, ret, "F_DUPFD_CLOEXEC", rc);
 		if (ret > -1)
 			(void)close(ret);
@@ -904,7 +904,7 @@ tidy:
 
 const stressor_info_t stress_fcntl_info = {
 	.stressor = stress_fcntl,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
