@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -316,12 +316,16 @@ static int stress_fstat(stress_args_t *args)
 		if (do_not_stat(path))
 			continue;
 		if ((si = (stress_stat_info_t *)calloc(1, sizeof(*si))) == NULL) {
-			pr_err("%s: out of memory\n", args->name);
+			pr_err("%s: out of memory allocating %zu bytes%s\n",
+				args->name, sizeof(*si),
+				stress_get_memfree_str());
 			(void)closedir(dp);
 			goto free_cache;
 		}
-		if ((si->path = strdup(path)) == NULL) {
-			pr_err("%s: out of memory\n", args->name);
+		if ((si->path = shim_strdup(path)) == NULL) {
+			pr_err("%s: out of memory allocating %zu bytes%s\n",
+				args->name, strlen(path),
+				stress_get_memfree_str());
 			free(si);
 			(void)closedir(dp);
 			goto free_cache;
@@ -377,7 +381,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_fstat_info = {
 	.stressor = stress_fstat,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
