@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -127,7 +127,9 @@ static int heapsort_nonlibc(
 }
 
 static const stress_heapsort_method_t stress_heapsort_methods[] = {
-#if defined(HAVE_LIB_BSD)
+#if (defined(HAVE_LIB_BSD) &&		\
+     defined(HAVE_BSD_STDLIB_H)) ||	\
+     defined(HAVE_HEAPSORT)
 	{ "heapsort-libc",	heapsort },
 #endif
 	{ "heapsort-nonlibc",	heapsort_nonlibc },
@@ -140,7 +142,7 @@ static const char *stress_heapsort_method(const size_t i)
 
 static const stress_opt_t opts[] = {
 	{ OPT_heapsort_size,   "heapsort-size",   TYPE_ID_UINT64, MIN_HEAPSORT_SIZE, MAX_HEAPSORT_SIZE, NULL },
-	{ OPT_heapsort_method, "heapsort-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_heapsort_method },
+	{ OPT_heapsort_method, "heapsort-method", TYPE_ID_SIZE_T_METHOD, 0, 0, (void *)stress_heapsort_method },
 	END_OPT,
 };
 
@@ -326,8 +328,9 @@ tidy:
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "heapsort comparisons per sec",
 		rate, STRESS_METRIC_HARMONIC_MEAN);
+	rate = (sorted > 0.0) ? count / sorted : 0.0;
 	stress_metrics_set(args, 1, "heapsort comparisons per item",
-		count / sorted, STRESS_METRIC_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	pr_dbg("%s: %.2f heapsort comparisons per sec\n", args->name, rate);
 
