@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2025 Colin Ian King
+ * Copyright (C) 2021-2026 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-mmap.h"
 #include "core-numa.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H) &&  \
@@ -48,7 +49,8 @@ static const stress_opt_t opts[] = {
 	END_OPT,
 };
 
-#if (((defined(HAVE_COMPILER_GCC_OR_MUSL) ||		\
+#if (defined(HAVE_SIGLONGJMP) &&			\
+     ((defined(HAVE_COMPILER_GCC_OR_MUSL) ||		\
        defined(HAVE_COMPILER_CLANG) ||			\
        defined(HAVE_COMPILER_ICC) ||			\
        defined(HAVE_COMPILER_ICX) ||			\
@@ -90,8 +92,9 @@ do {							\
 } while (0)
 #endif
 
-#define BUFFER_SIZE	(1024 * 1024 * 16)
-#define CHUNK_SIZE	(64 * 4)
+#define BUFFER_SIZE		(1024 * 1024 * 8)
+#define SHARED_BUFFER_SIZE	(1024 * 1024 * 8)
+#define CHUNK_SIZE		(64 * 4)
 
 #if defined(HAVE_SYNC_BOOL_COMPARE_AND_SWAP)
 /* basically locked cmpxchg */
@@ -106,7 +109,7 @@ do {								\
 	(void)ptr;						\
 	(void)old_val;						\
 	(void)new_val;						\
-}
+} while (0)
 #endif
 
 #define MEM_LOCK_AND_INC(ptr, inc)		\
@@ -145,6 +148,25 @@ static bool do_misaligned;
 static bool do_splitlock;
 #endif
 static bool do_sigill;
+static uint32_t *shared_buffer = (uint32_t *)MAP_FAILED;
+
+static void stress_lockbus_init(const uint32_t instances)
+{
+	(void)instances;
+
+	shared_buffer = (uint32_t *)stress_mmap_populate(NULL, SHARED_BUFFER_SIZE,
+		PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
+	if (shared_buffer != MAP_FAILED)
+		stress_set_vma_anon_name(shared_buffer, SHARED_BUFFER_SIZE, "lockbus-shared-data");
+}
+
+static void stress_lockbus_deinit(void)
+{
+	if (shared_buffer != MAP_FAILED) {
+		(void)munmap((void *)shared_buffer, SHARED_BUFFER_SIZE);
+		shared_buffer = (uint32_t *)MAP_FAILED;
+	}
+}
 
 static void NORETURN MLOCKED_TEXT stress_sigill_handler(int signum)
 {
@@ -157,6 +179,7 @@ static void NORETURN MLOCKED_TEXT stress_sigill_handler(int signum)
 	}
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 static void NORETURN MLOCKED_TEXT stress_sigbus_misaligned_handler(int signum)
@@ -166,6 +189,7 @@ static void NORETURN MLOCKED_TEXT stress_sigbus_misaligned_handler(int signum)
 	do_misaligned = false;
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 #if defined(STRESS_ARCH_X86)
@@ -176,6 +200,7 @@ static void NORETURN MLOCKED_TEXT stress_sigbus_splitlock_handler(int signum)
 	do_splitlock = false;
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 #endif
 
@@ -194,6 +219,9 @@ static int stress_lockbus(stress_args_t *args)
 	timer_t timerid;
 	NOCLOBBER int timer_ret = -1;
 #endif
+#if defined(HAVE_NUMA_LOCKBUS)
+	NOCLOBBER stress_numa_mask_t *numa_mask;
+#endif
 #if defined(STRESS_ARCH_X86)
 	uint32_t *splitlock_ptr1, *splitlock_ptr2;
 	bool lockbus_nosplit = false;
@@ -204,32 +232,46 @@ static int stress_lockbus(stress_args_t *args)
 	if (stress_sighandler(args->name, SIGBUS, stress_sigbus_splitlock_handler, NULL) < 0)
 		return EXIT_FAILURE;
 #endif
-#if defined(HAVE_NUMA_LOCKBUS)
-	NOCLOBBER stress_numa_mask_t *numa_mask;
-#endif
 
-	buffer = (uint32_t*)stress_mmap_populate(NULL, BUFFER_SIZE,
+	if (shared_buffer == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %zu shared bytes%s, errno=%d (%s), skipping stressor\n",
+			args->name, (size_t)SHARED_BUFFER_SIZE,
+			stress_get_memfree_str(), errno, strerror(errno));
+		return EXIT_NO_RESOURCE;
+	}
+
+	buffer = (uint32_t *)stress_mmap_populate(NULL, BUFFER_SIZE,
 			PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (buffer == MAP_FAILED) {
-		rc = stress_exit_status(errno);
-
-		pr_err("%s: mmap failed\n", args->name);
-		return rc;
+		pr_inf_skip("%s: failed to mmap %zu bytes%s, errno=%d (%s), skipping stressor\n",
+			args->name, (size_t)BUFFER_SIZE,
+			stress_get_memfree_str(), errno, strerror(errno));
+		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(buffer, BUFFER_SIZE, "lockbus-data");
 
+#if defined(STRESS_ARCH_M68K)
+	do_misaligned = false;
+#else
 	do_misaligned = true;
 	misaligned_ptr1 = (uint32_t *)(uintptr_t)((uint8_t *)buffer + 1);
 	misaligned_ptr2 = (uint32_t *)(uintptr_t)((uint8_t *)buffer + 10);
-
-	if (stress_sighandler(args->name, SIGBUS, stress_sigbus_misaligned_handler, NULL) < 0)
-		return EXIT_FAILURE;
-#if defined(HAVE_TIMER_FUNCS)
-	if (stress_sighandler(args->name, SIGRTMIN, stress_sigbus_misaligned_handler, NULL) < 0)
-		return EXIT_FAILURE;
 #endif
-	if (stress_sighandler(args->name, SIGILL, stress_sigill_handler, NULL) < 0)
-		return EXIT_FAILURE;
+
+	if (stress_sighandler(args->name, SIGBUS, stress_sigbus_misaligned_handler, NULL) < 0) {
+		rc = EXIT_FAILURE;
+		goto unmap_buffer;
+	}
+#if defined(HAVE_TIMER_FUNCS)
+	if (stress_sighandler(args->name, SIGRTMIN, stress_sigbus_misaligned_handler, NULL) < 0) {
+		rc = EXIT_FAILURE;
+		goto unmap_buffer;
+	}
+#endif
+	if (stress_sighandler(args->name, SIGILL, stress_sigill_handler, NULL) < 0) {
+		rc = EXIT_FAILURE;
+		goto unmap_buffer;
+	}
 	if (sigsetjmp(jmp_env, 1))
 		goto misaligned_done;
 
@@ -263,9 +305,12 @@ static int stress_lockbus(stress_args_t *args)
 		}
 	}
 #endif
+
+#if !defined(STRESS_ARCH_M68K)
 	/* These can hang on old ppc64 linux kernels */
 	MEM_LOCK_AND_INC(misaligned_ptr1, 1);
 	MEM_LOCK_AND_INC(misaligned_ptr2, 1);
+#endif
 
 misaligned_done:
 #if defined(HAVE_TIMER_FUNCS)
@@ -281,7 +326,7 @@ misaligned_done:
 		rc = EXIT_NO_RESOURCE;
 		goto done;
 	}
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: misaligned splitlocks %s\n", args->name,
 			do_misaligned ? "enabled" : "disabled");
 
@@ -293,7 +338,7 @@ misaligned_done:
 	/* Split lock on a cache boundary */
 	splitlock_ptr2 = (uint32_t *)(uintptr_t)(((uint8_t *)buffer) + 64 - (sizeof(*splitlock_ptr2) >> 1));
 	do_splitlock = !lockbus_nosplit;
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: splitlocks %s\n", args->name,
 			do_splitlock ? "enabled" : "disabled");
 	if (sigsetjmp(jmp_env, 1) && !stress_continue(args))
@@ -307,8 +352,10 @@ misaligned_done:
 
 		numa_nodes = stress_numa_mask_alloc();
 		if (numa_nodes) {
-			if (stress_numa_mask_nodes_get(numa_nodes) > 0)
-				stress_numa_randomize_pages(args, numa_nodes, numa_mask, buffer, args->page_size, BUFFER_SIZE);
+			if (stress_numa_mask_nodes_get(numa_nodes) > 0) {
+				stress_numa_randomize_pages(args, numa_nodes, numa_mask, buffer, BUFFER_SIZE, args->page_size);
+				stress_numa_randomize_pages(args, numa_nodes, numa_mask, shared_buffer, SHARED_BUFFER_SIZE, args->page_size);
+			}
 			stress_numa_mask_free(numa_nodes);
 		}
 		stress_numa_mask_free(numa_mask);
@@ -322,7 +369,9 @@ misaligned_done:
 	duration = 0;
 	count = 0;
 	do {
-		uint32_t *ptr0 = buffer + (stress_mwc32modn(BUFFER_SIZE - CHUNK_SIZE) >> 2);
+		uint32_t *ptr0 = stress_mwc1() ?
+			buffer + (stress_mwc32modn(BUFFER_SIZE - CHUNK_SIZE) >> 2) :
+			shared_buffer + (stress_mwc32modn(SHARED_BUFFER_SIZE - CHUNK_SIZE) >> 2);
 #if defined(STRESS_ARCH_X86)
 		uint32_t *ptr1 = do_splitlock ? splitlock_ptr1 : ptr0;
 		uint32_t *ptr2 = do_splitlock ? splitlock_ptr2 : ptr0;
@@ -386,6 +435,7 @@ done:
 	stress_metrics_set(args, 0, "nanosecs per memory lock operation",
 		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 
+unmap_buffer:
 	(void)munmap((void *)buffer, BUFFER_SIZE);
 
 	return rc;
@@ -393,16 +443,18 @@ done:
 
 const stressor_info_t stress_lockbus_info = {
 	.stressor = stress_lockbus,
-	.class = CLASS_CPU_CACHE | CLASS_MEMORY,
+	.classifier = CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opts = opts,
-	.help = help
+	.help = help,
+	.init = stress_lockbus_init,
+	.deinit = stress_lockbus_deinit
 };
 #else
 const stressor_info_t stress_lockbus_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE | CLASS_MEMORY,
+	.classifier = CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opts = opts,
 	.help = help,
-	.unimplemented_reason = "built without gcc __atomic* lock builtins"
+	.unimplemented_reason = "built without gcc __atomic* lock builtins or siglongjmp support"
 };
 #endif
