@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,7 @@
 #include "core-builtin.h"
 #include "core-madvise.h"
 #include "core-mincore.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-pthread.h"
 
@@ -31,14 +32,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_madvise_hwpoison(const char *opt)
-{
-	return stress_set_setting_true("madvise-hwpoison", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_madvise_hwpoison,	stress_set_madvise_hwpoison },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_madvise_hwpoison,	"madvise-hwpoison", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_MADVISE)
@@ -53,7 +49,8 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 typedef struct madvise_ctxt {
 	stress_args_t *args;
 	void *buf;
-	char *smaps;
+	char smaps[PATH_MAX];
+	char maps[PATH_MAX];
 	size_t sz;
 	bool  is_thread;
 	bool  hwpoison;
@@ -62,136 +59,6 @@ typedef struct madvise_ctxt {
 static sigjmp_buf jmp_env;
 static uint64_t sigbus_count;
 
-static const int madvise_options[] = {
-#if defined(MADV_NORMAL)
-	MADV_NORMAL,
-#endif
-#if defined(MADV_RANDOM)
-	MADV_RANDOM,
-#endif
-#if defined(MADV_SEQUENTIAL)
-	MADV_SEQUENTIAL,
-#endif
-#if defined(MADV_WILLNEED)
-	MADV_WILLNEED,
-#endif
-#if defined(MADV_DONTNEED)
-	MADV_DONTNEED,
-#endif
-#if defined(MADV_REMOVE)
-	MADV_REMOVE,
-#endif
-#if defined(MADV_DONTFORK)
-	MADV_DONTFORK,
-#endif
-#if defined(MADV_DOFORK)
-	MADV_DOFORK,
-#endif
-#if defined(MADV_MERGEABLE)
-	MADV_MERGEABLE,
-#endif
-#if defined(MADV_UNMERGEABLE)
-	MADV_UNMERGEABLE,
-#endif
-#if defined(MADV_SOFT_OFFLINE)
-	MADV_SOFT_OFFLINE,
-#endif
-#if defined(MADV_HUGEPAGE)
-	MADV_HUGEPAGE,
-#endif
-#if defined(MADV_NOHUGEPAGE)
-	MADV_NOHUGEPAGE,
-#endif
-#if defined(MADV_DONTDUMP)
-	MADV_DONTDUMP,
-#endif
-#if defined(MADV_DODUMP)
-	MADV_DODUMP,
-#endif
-#if defined(MADV_FREE)
-	MADV_FREE,
-#endif
-#if defined(MADV_HWPOISON)
-	MADV_HWPOISON,
-#endif
-#if defined(MADV_WIPEONFORK)
-	MADV_WIPEONFORK,
-#endif
-#if defined(MADV_KEEPONFORK)
-	MADV_KEEPONFORK,
-#endif
-#if defined(MADV_INHERIT_ZERO)
-	MADV_INHERIT_ZERO,
-#endif
-#if defined(MADV_COLD)
-	MADV_COLD,
-#endif
-#if defined(MADV_PAGEOUT)
-	MADV_PAGEOUT,
-#endif
-#if defined(MADV_POPULATE_READ)
-	MADV_POPULATE_READ,
-#endif
-#if defined(MADV_POPULATE_WRITE)
-	MADV_POPULATE_WRITE,
-#endif
-#if defined(MADV_DONTNEED_LOCKED)
-	MADV_DONTNEED_LOCKED,
-#endif
-/* Linux 6.0 */
-#if defined(MADV_COLLAPSE)
-	MADV_COLLAPSE,
-#endif
-/* FreeBSD */
-#if defined(MADV_AUTOSYNC)
-	MADV_AUTOSYNC,
-#endif
-/* FreeBSD and DragonFlyBSD */
-#if defined(MADV_CORE)
-	MADV_CORE,
-#endif
-/* FreeBSD */
-#if defined(MADV_PROTECT)
-	MADV_PROTECT,
-#endif
-/* Linux 5.14 */
-#if defined(MADV_POPULATE_READ)
-	MADV_POPULATE_READ,
-#endif
-/* Linux 5.14 */
-#if defined(MADV_POPULATE_WRITE)
-	MADV_POPULATE_WRITE,
-#endif
-/* OpenBSD */
-#if defined(MADV_SPACEAVAIL)
-	MADV_SPACEAVAIL,
-#endif
-/* OS X */
-#if defined(MADV_ZERO_WIRED_PAGES)
-	MADV_ZERO_WIRED_PAGES,
-#endif
-/* Solaris */
-#if defined(MADV_ACCESS_DEFAULT)
-	MADV_ACCESS_DEFAULT,
-#endif
-/* Solaris */
-#if defined(MADV_ACCESS_LWP)
-	MADV_ACCESS_LWP,
-#endif
-/* Solaris */
-#if defined(MADV_ACCESS_MANY)
-	MADV_ACCESS_MANY,
-#endif
-/* DragonFlyBSD */
-#if defined(MADV_INVAL)
-	MADV_INVAL,
-#endif
-/* DragonFlyBSD */
-#if defined(MADV_NOCORE)
-	MADV_NOCORE,
-#endif
-};
-
 /*
  *  stress_sigbus_handler()
  *     SIGBUS handler
@@ -203,34 +70,50 @@ static void NORETURN MLOCKED_TEXT stress_sigbus_handler(int signum)
 	sigbus_count++;
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
+}
+
+/*
+ *  stress_read_proc_file()
+ *	read smaps file for extra kernel exercising
+ */
+static void stress_read_proc_file(const char *filename, bool *ignore)
+{
+	int fd;
+
+	if (*ignore)
+		return;
+
+	fd = open(filename, O_RDONLY);
+	if (fd < 0) {
+		*ignore = true;
+		return;
+	}
+	(void)stress_read_discard(fd);
+	(void)close(fd);
 }
 
-#if defined(MADV_FREE)
 /*
  *  stress_read_proc_smaps()
  *	read smaps file for extra kernel exercising
  */
-static void stress_read_proc_smaps(const char *smaps)
+static inline void stress_read_proc_smaps(const char *smaps)
 {
 	static bool ignore = false;
-	ssize_t ret;
-	char buffer[4096];
-	int fd;
 
-	if (ignore)
-		return;
+	stress_read_proc_file(smaps, &ignore);
+}
 
-	fd = open(smaps, O_RDONLY);
-	if (fd < 0) {
-		ignore = true;
-		return;
-	}
-	do {
-		ret = read(fd, buffer, sizeof(buffer));
-	} while (ret == (ssize_t)sizeof(buffer));
-	(void)close(fd);
+/*
+ *  stress_read_proc_maps()
+ *	read maps file for extra kernel exercising
+ */
+static inline void stress_read_proc_maps(const char *maps)
+{
+	static bool ignore = false;
+
+	stress_read_proc_file(maps, &ignore);
 }
-#endif
 
 /*
  *  stress_random_advise()
@@ -242,8 +125,7 @@ static int stress_random_advise(
 	const size_t size,
 	const bool hwpoison)
 {
-	const int idx = stress_mwc32modn((size_t)SIZEOF_ARRAY(madvise_options));
-	const int advise = madvise_options[idx];
+	int advice = madvise_options[stress_mwcsizemodn(madvise_options_elements)];
 #if defined(MADV_HWPOISON) || defined(MADV_SOFT_OFFLINE)
 	static int hwpoison_count = 0;
 #if defined(MADV_NORMAL)
@@ -253,8 +135,10 @@ static int stress_random_advise(
 #endif
 #endif
 
+	advice = stress_advice_check(advice);
+
 #if defined(MADV_HWPOISON)
-	if (advise == MADV_HWPOISON) {
+	if (advice == MADV_HWPOISON) {
 		if (hwpoison) {
 			const size_t page_size = args->page_size;
 			const size_t vec_size = (size + page_size - 1) / page_size;
@@ -273,7 +157,7 @@ static int stress_random_advise(
 			}
 
 			vec = (unsigned char *)calloc(vec_size, sizeof(*vec));
-			if (vec) {
+			if (LIKELY(vec != NULL)) {
 				size_t i;
 				int ret;
 
@@ -322,8 +206,8 @@ static int stress_random_advise(
 #endif
 
 #if defined(MADV_SOFT_OFFLINE)
-	if (advise == MADV_SOFT_OFFLINE) {
-		static int soft_offline_count;
+	if (advice == MADV_SOFT_OFFLINE) {
+		static int soft_offline_count = 0;
 
 		/* ..and minimize number of soft offline pages */
 		if ((soft_offline_count >= NUM_SOFT_OFFLINE_MAX) ||
@@ -332,7 +216,7 @@ static int stress_random_advise(
 		soft_offline_count++;
 	}
 #endif
-	return advise;
+	return advice;
 }
 
 /*
@@ -347,36 +231,56 @@ static void *stress_madvise_pages(void *arg)
 	void *buf = ctxt->buf;
 	const size_t sz = ctxt->sz;
 	const size_t page_size = args->page_size;
-	static void *nowt = NULL;
 
 	if (ctxt->is_thread) {
 		sigset_t set;
 
-		sigemptyset(&set);
-		sigaddset(&set, SIGBUS);
-
+		(void)sigemptyset(&set);
+		(void)sigaddset(&set, SIGBUS);
 		(void)pthread_sigmask(SIG_SETMASK, &set, NULL);
+
+		stress_random_small_sleep();
 	}
 
 	for (n = 0; n < sz; n += page_size) {
 		void *ptr = (void *)(((uint8_t *)buf) + n);
-		const int advise = stress_random_advise(args, ptr, page_size, ctxt->hwpoison);
+		const int advice = stress_random_advise(args, ptr, page_size, ctxt->hwpoison);
 
-		(void)shim_madvise(ptr, page_size, advise);
-#if defined(MADV_FREE)
-		if (advise == MADV_FREE)
+		(void)shim_madvise(ptr, page_size, advice);
+		if (stress_mwc8() < 16)
 			stress_read_proc_smaps(ctxt->smaps);
+		if (stress_mwc8() < 16)
+			stress_read_proc_maps(ctxt->maps);
+#if defined(MADV_GUARD_INSTALL) && defined(MADV_NORMAL)
+		/* avoid segfaults by setting back to normal */
+		if (advice == MADV_GUARD_INSTALL)
+			(void)shim_madvise(ptr, page_size, MADV_NORMAL);
 #endif
 		(void)shim_msync(ptr, page_size, MS_ASYNC);
 	}
 	for (n = 0; n < sz; n += page_size) {
-		size_t m = (size_t)(stress_mwc64modn_maybe_pwr2((uint64_t)sz) & ~(page_size - 1));
+		const size_t m = (size_t)(stress_mwc64modn((uint64_t)sz) & ~(page_size - 1));
 		void *ptr = (void *)(((uint8_t *)buf) + m);
-		const int advise = stress_random_advise(args, ptr, page_size, ctxt->hwpoison);
+		const int advice = stress_random_advise(args, ptr, page_size, ctxt->hwpoison);
 
-		(void)shim_madvise(ptr, page_size, advise);
+		(void)shim_madvise(ptr, page_size, advice);
+#if defined(MADV_GUARD_INSTALL) && defined(MADV_NORMAL)
+		/* avoid segfaults by setting back to normal */
+		if (advice == MADV_GUARD_INSTALL)
+			(void)shim_madvise(ptr, page_size, MADV_NORMAL);
+#endif
 		(void)shim_msync(ptr, page_size, MS_ASYNC);
 	}
+#if defined(MADV_PAGEOUT) && defined(MS_SYNC)
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE) {
+		(void)shim_madvise(buf, sz, MADV_PAGEOUT);
+		(void)shim_msync(buf, sz, MS_SYNC);
+#if defined(MADV_POPULATE_WRITE)
+		(void)shim_madvise(buf, sz, MADV_POPULATE_WRITE);
+		(void)shim_msync(buf, sz, MS_SYNC);
+#endif
+	}
+#endif
 
 	/*
 	 *  Exercise a highly likely bad advice option
@@ -422,13 +326,13 @@ static void *stress_madvise_pages(void *arg)
 		unmapped = mmap(NULL, page_size, PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 		if (unmapped != MAP_FAILED) {
-			(void)stress_munmap_retry_enomem(unmapped, page_size);
+			(void)stress_munmap_force(unmapped, page_size);
 			(void)shim_madvise(unmapped, page_size, MADV_NORMAL);
 		}
 	}
 #endif
 
-	return &nowt;
+	return &g_nowt;
 }
 
 static void stress_process_madvise(const pid_t pid, void *buf, const size_t sz)
@@ -477,10 +381,10 @@ static int stress_madvise(stress_args_t *args)
 	const size_t sz = (4 *  MB) & ~(page_size - 1);
 	const pid_t pid = getpid();
 	int fd = -1;
+	NOCLOBBER size_t advice = 0;
 	NOCLOBBER int ret;
 	NOCLOBBER int num_mem_retries;
 	char filename[PATH_MAX];
-	char smaps[PATH_MAX];
 	char *page;
 	size_t n;
 	madvise_ctxt_t ctxt;
@@ -490,7 +394,7 @@ static int stress_madvise(stress_args_t *args)
 	NOCLOBBER uint8_t madv_tries;
 #endif
 
-	(void)memset(&ctxt, 0, sizeof(ctxt));
+	(void)shim_memset(&ctxt, 0, sizeof(ctxt));
 	(void)stress_get_setting("madvise-hwpoison", &ctxt.hwpoison);
 
 	num_mem_retries = 0;
@@ -503,12 +407,16 @@ static int stress_madvise(stress_args_t *args)
 	page = (char *)stress_mmap_populate(NULL, page_size, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate %zd byte page, skipping stressor\n",
-			args->name, page_size);
+		pr_inf_skip("%s: failed to mmap %zu byte page%s, errno=%d (%s), skipping stressor\n",
+			args->name, page_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(page, page_size, "data-page");
 
-	(void)snprintf(smaps, sizeof(smaps), "/proc/%" PRIdMAX "/smaps", (intmax_t)pid);
+	/* Don't use /proc/self/, use more direct /proc/$PID/ */
+	(void)snprintf(ctxt.smaps, sizeof(ctxt.smaps), "/proc/%" PRIdMAX "/smaps", (intmax_t)pid);
+	(void)snprintf(ctxt.maps, sizeof(ctxt.maps), "/proc/%" PRIdMAX "/maps", (intmax_t)pid);
 
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
@@ -554,19 +462,21 @@ static int stress_madvise(stress_args_t *args)
 		VOID_RET(ssize_t, write(fd, page, page_size));
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		NOCLOBBER uint8_t *buf;
 		NOCLOBBER bool file_mapped;
 
-		if (num_mem_retries >= NUM_MEM_RETRIES_MAX) {
+		if (UNLIKELY(num_mem_retries >= NUM_MEM_RETRIES_MAX)) {
 			pr_err("%s: gave up trying to mmap, no available memory\n",
 				args->name);
 			break;
 		}
 
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		file_mapped = stress_mwc1();
@@ -575,7 +485,7 @@ static int stress_madvise(stress_args_t *args)
 								MAP_PRIVATE, fd, 0);
 		} else {
 			buf = (uint8_t *)stress_mmap_populate(NULL, sz, PROT_READ | PROT_WRITE,
-								MAP_ANONYMOUS, -1, 0);
+								MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 		}
 		if (buf == MAP_FAILED) {
 			/* Force MAP_POPULATE off, just in case */
@@ -592,14 +502,13 @@ static int stress_madvise(stress_args_t *args)
 		}
 
 		(void)shim_memset(buf, 0xff, sz);
-		(void)stress_madvise_random(buf, sz);
+		(void)stress_madvise_randomize(buf, sz);
 		(void)stress_mincore_touch_pages(buf, sz);
 		stress_process_madvise(pid, buf, sz);
 
 		ctxt.args = args;
 		ctxt.buf = buf;
 		ctxt.sz = sz;
-		ctxt.smaps = smaps;
 
 #if defined(HAVE_LIB_PTHREAD)
 		{
@@ -649,7 +558,7 @@ static int stress_madvise(stress_args_t *args)
 			val = stress_mwc8();
 
 			for (n = 0; n < sz; n += page_size) {
-				register uint8_t v = (uint8_t)(val + n);
+				register const uint8_t v = (uint8_t)(val + n);
 
 				buf[n] = v;
 			}
@@ -661,7 +570,7 @@ static int stress_madvise(stress_args_t *args)
 				goto madv_free_out;
 
 			for (n = 0; n < sz; n += page_size) {
-				register uint8_t v = (uint8_t)(val + n);
+				register const uint8_t v = (uint8_t)(val + n);
 
 				if (buf[n] != v)
 					madv_frees_raced++;
@@ -683,6 +592,15 @@ madv_free_out:
 			(void)madvise(bad_addr, page_size * 2, MADV_NORMAL);
 		}
 #endif
+
+		/*
+		 * Some systems allow zero sized page zero madvise
+		 * to see if that madvice is implemented, so try this
+		 */
+		(void)madvise(0, 0, stress_advice_check(madvise_options[advice]));
+		advice++;
+		advice = (advice >= madvise_options_elements) ? 0: advice;
+
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
@@ -704,18 +622,17 @@ madv_free_out:
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_madvise_info = {
+const stressor_info_t stress_madvise_info = {
 	.stressor = stress_madvise,
-	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_VM | CLASS_OS,
+	.opts = opts,
 	.help = help
 };
 #else
-stressor_info_t stress_madvise_info = {
+const stressor_info_t stress_madvise_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
-	.help = help,
-	.unimplemented_reason = "built without madvise() system call"
+	.classifier = CLASS_VM | CLASS_OS,
+	.opts = opts,
+	.help = help
 };
 #endif
