@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2021-2025 Colin Ian King.
+ * Copyright (C) 2021-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,15 +18,18 @@
  *
  */
 #include "stress-ng.h"
+#include "core-asm-x86.h"
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-nt-store.h"
 #include "core-out-of-memory.h"
 #include "core-target-clones.h"
 #include "core-vecmath.h"
 
 #include <time.h>
+#include <math.h>
 
 #define MR_RD			(0x0001)
 #define MR_WR			(0x0002)
@@ -51,14 +54,16 @@ static const stress_help_t help[] = {
 };
 
 #if defined(HAVE_VECMATH)
-typedef uint64_t stress_uint32w1024_t	__attribute__ ((vector_size(1024 / 8)));
-typedef uint64_t stress_uint32w512_t	__attribute__ ((vector_size(512 / 8)));
-typedef uint64_t stress_uint32w256_t	__attribute__ ((vector_size(256 / 8)));
-typedef uint64_t stress_uint32w128_t	__attribute__ ((vector_size(128 / 8)));
+typedef uint64_t stress_uint64w1024_t	__attribute__ ((vector_size(1024 / 8)));
+typedef uint64_t stress_uint64w512_t	__attribute__ ((vector_size(512 / 8)));
+typedef uint64_t stress_uint64w256_t	__attribute__ ((vector_size(256 / 8)));
+typedef uint64_t stress_uint64w128_t	__attribute__ ((vector_size(128 / 8)));
 #endif
 
+#if defined(HAVE_SIGLONGJMP)
 static volatile bool do_jmp = true;
 static sigjmp_buf jmpbuf;
+#endif
 
 typedef struct {
 	double		duration;
@@ -86,6 +91,7 @@ typedef struct {
 	const stress_memrate_func_t	func_rate;
 } stress_memrate_info_t;
 
+#if defined(HAVE_SIGLONGJMP)
 static void stress_memrate_alarm_handler(int signum)
 {
         (void)signum;
@@ -93,8 +99,10 @@ static void stress_memrate_alarm_handler(int signum)
 	if (do_jmp) {
 		do_jmp = false;
 	        siglongjmp(jmpbuf, 1);
+		stress_no_return();
 	}
 }
+#endif
 
 static uint64_t stress_memrate_loops(
 	const stress_memrate_context_t *context,
@@ -134,7 +142,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_read##size(		\
 	bool *valid)						\
 {								\
 	register type v, *ptr;					\
-	void *start ALIGNED(4096) = context->start;		\
+	type *start ALIGNED(4096) = (type *)context->start;	\
 	const type *end ALIGNED(4096) = (type *)context->end;	\
 								\
 	for (ptr = start; ptr < end;) {				\
@@ -184,7 +192,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_read_rate##size(		\
 {								\
 	register type *ptr;					\
 	type v;							\
-	void *start ALIGNED(4096) = context->start;		\
+	type *start ALIGNED(4096) = (type *)context->start;	\
 	const type *end ALIGNED(4096) = (type *)context->end;	\
 	const uint64_t loops = 					\
 		stress_memrate_loops(context, sizeof(type) * 16);\
@@ -259,14 +267,14 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_read_rate##size(		\
 #define no_prefetch(ptr, arg1, arg2)
 
 #if defined(HAVE_VECMATH)
-STRESS_MEMRATE_READ(1024, stress_uint32w1024_t, no_prefetch)
-STRESS_MEMRATE_READ_RATE(1024, stress_uint32w1024_t, no_prefetch)
-STRESS_MEMRATE_READ(512, stress_uint32w512_t, no_prefetch)
-STRESS_MEMRATE_READ_RATE(512, stress_uint32w512_t, no_prefetch)
-STRESS_MEMRATE_READ(256, stress_uint32w256_t, no_prefetch)
-STRESS_MEMRATE_READ_RATE(256, stress_uint32w256_t, no_prefetch)
-STRESS_MEMRATE_READ(128, stress_uint32w128_t, no_prefetch)
-STRESS_MEMRATE_READ_RATE(128, stress_uint32w128_t, no_prefetch)
+STRESS_MEMRATE_READ(1024, stress_uint64w1024_t, no_prefetch)
+STRESS_MEMRATE_READ_RATE(1024, stress_uint64w1024_t, no_prefetch)
+STRESS_MEMRATE_READ(512, stress_uint64w512_t, no_prefetch)
+STRESS_MEMRATE_READ_RATE(512, stress_uint64w512_t, no_prefetch)
+STRESS_MEMRATE_READ(256, stress_uint64w256_t, no_prefetch)
+STRESS_MEMRATE_READ_RATE(256, stress_uint64w256_t, no_prefetch)
+STRESS_MEMRATE_READ(128, stress_uint64w128_t, no_prefetch)
+STRESS_MEMRATE_READ_RATE(128, stress_uint64w128_t, no_prefetch)
 #endif
 #if defined(HAVE_INT128_T) && !defined(HAVE_VECMATH)
 STRESS_MEMRATE_READ(128, __uint128_t, no_prefetch)
@@ -363,7 +371,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3	stress_memrate_write##size(	\
 	const stress_memrate_context_t *context,		\
 	bool *valid)						\
 {								\
-	void *start ALIGNED(4096) = context->start;		\
+	type *start ALIGNED(4096) = (type *)context->start;	\
 	const type *end ALIGNED(4096) = (type *)context->end;	\
 	register type v, *ptr;					\
 								\
@@ -418,7 +426,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_stos(
 		func((void *)ptr, loops);
 	}
 	/* And any residual less than chunk_size .. */
-	loops = (uint32_t)((end - ptr)/ wr_size);
+	loops = (uint32_t)((end - ptr) / wr_size);
 	if (loops) {
 		func((void *)ptr, loops);
 		ptr = end;
@@ -464,7 +472,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_stos_rate(
 	}
 
 	/* And any residual less than chunk_size .. */
-	loops = (uint32_t)((end - ptr)/ wr_size);
+	loops = (uint32_t)((end - ptr) / wr_size);
 	if (loops) {
 		func((void *)ptr, loops);
 		t2 = stress_time_now();
@@ -626,7 +634,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_write_rate##size(	\
 	const stress_memrate_context_t *context,		\
 	bool *valid)						\
 {								\
-	void *start ALIGNED(4096) = context->start;		\
+	type *start ALIGNED(4096) = (type *)context->start;	\
 	const type *end ALIGNED(4096) = (type *)context->end;	\
 	const uint64_t loops = 					\
 		stress_memrate_loops(context, sizeof(type) * 16);\
@@ -689,22 +697,16 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_write_rate##size(	\
 	return ((uintptr_t)ptr - (uintptr_t)start) / KB;	\
 }
 
-/*
- *
- * See https://akkadia.org/drepper/cpumemory.pdf - section 6.1
- *  non-temporal writes using movntdq. Data is not going to be
- *  read, so no need to cache. Write directly to memory.
- */
-#define STRESS_MEMRATE_WRITE_NT(size, type, op)			\
-static uint64_t OPTIMIZE3 stress_memrate_write_nt##size(	\
+#define STRESS_MEMRATE_WRITE_OP(size, type, op, write_op, check)	\
+static uint64_t OPTIMIZE3 stress_memrate_write_ ## write_op ## size (	\
 	const stress_memrate_context_t *context,		\
 	bool *valid)						\
 {								\
-	void *start ALIGNED(4096) = context->start;		\
+	type *start ALIGNED(4096) = (type *)context->start;	\
 	const type *end ALIGNED(4096) = (type *)context->end;	\
 	register type v, *ptr;					\
 								\
-	if (!stress_cpu_x86_has_sse2()) {			\
+	if (!check()) {						\
 		*valid = false;					\
 		return 0;					\
 	}							\
@@ -741,12 +743,12 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt##size(	\
 	return ((uintptr_t)ptr - (uintptr_t)start) / KB;	\
 }
 
-#define STRESS_MEMRATE_WRITE_NT_RATE(size, type, op)		\
-static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
+#define STRESS_MEMRATE_WRITE_OP_RATE(size, type, op, write_op, check)	\
+static uint64_t OPTIMIZE3 stress_memrate_write_ ## write_op ## _rate ## size( \
 	const stress_memrate_context_t *context,		\
 	bool *valid)						\
 {								\
-	void *start ALIGNED(4096) = context->start;		\
+	type *start ALIGNED(4096) = (type *)context->start;	\
 	const type *end ALIGNED(4096) = (type *)context->end;	\
 	const uint64_t loops = 					\
 		stress_memrate_loops(context, sizeof(type) * 16);\
@@ -757,7 +759,7 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 		(MB * (double)context->memrate_wr_mbs);		\
 	register type v, *ptr;					\
 								\
-	if (!stress_cpu_x86_has_sse2()) {			\
+	if (!check()) {						\
 		*valid = false;					\
 		return 0;					\
 	}							\
@@ -816,30 +818,41 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 	return ((uintptr_t)ptr - (uintptr_t)start) / KB;	\
 }
 
+/*
+ * See https://akkadia.org/drepper/cpumemory.pdf - section 6.1
+ *  non-temporal writes using movntdq. Data is not going to be
+ *  read, so no need to cache. Write directly to memory.
+ */
 #if defined(HAVE_NT_STORE128)
-STRESS_MEMRATE_WRITE_NT(128, __uint128_t, stress_nt_store128)
-STRESS_MEMRATE_WRITE_NT_RATE(128, __uint128_t, stress_nt_store128)
+STRESS_MEMRATE_WRITE_OP(128, __uint128_t, stress_nt_store128, nt, stress_cpu_x86_has_sse2)
+STRESS_MEMRATE_WRITE_OP_RATE(128, __uint128_t, stress_nt_store128, nt, stress_cpu_x86_has_sse2)
 #endif
 
 #if defined(HAVE_NT_STORE64)
-STRESS_MEMRATE_WRITE_NT(64, uint64_t, stress_nt_store64)
-STRESS_MEMRATE_WRITE_NT_RATE(64, uint64_t, stress_nt_store64)
+STRESS_MEMRATE_WRITE_OP(64, uint64_t, stress_nt_store64, nt, stress_cpu_x86_has_sse2)
+STRESS_MEMRATE_WRITE_OP_RATE(64, uint64_t, stress_nt_store64, nt, stress_cpu_x86_has_sse2)
 #endif
 
 #if defined(HAVE_NT_STORE32)
-STRESS_MEMRATE_WRITE_NT(32, uint32_t, stress_nt_store32)
-STRESS_MEMRATE_WRITE_NT_RATE(32, uint32_t, stress_nt_store32)
+STRESS_MEMRATE_WRITE_OP(32, uint32_t, stress_nt_store32, nt, stress_cpu_x86_has_sse2)
+STRESS_MEMRATE_WRITE_OP_RATE(32, uint32_t, stress_nt_store32, nt, stress_cpu_x86_has_sse2)
+#endif
+
+#if defined(HAVE_ASM_X86_MOVDIRI) &&	\
+    defined(STRESS_ARCH_X86_64)
+STRESS_MEMRATE_WRITE_OP(64, uint64_t, stress_ds_store64, ds, stress_cpu_x86_has_movdiri)
+STRESS_MEMRATE_WRITE_OP_RATE(64, uint64_t, stress_ds_store64, ds, stress_cpu_x86_has_movdiri)
 #endif
 
 #if defined(HAVE_VECMATH)
-STRESS_MEMRATE_WRITE(1024, stress_uint32w1024_t)
-STRESS_MEMRATE_WRITE_RATE(1024, stress_uint32w1024_t)
-STRESS_MEMRATE_WRITE(512, stress_uint32w512_t)
-STRESS_MEMRATE_WRITE_RATE(512, stress_uint32w512_t)
-STRESS_MEMRATE_WRITE(256, stress_uint32w256_t)
-STRESS_MEMRATE_WRITE_RATE(256, stress_uint32w256_t)
-STRESS_MEMRATE_WRITE(128, stress_uint32w128_t)
-STRESS_MEMRATE_WRITE_RATE(128, stress_uint32w128_t)
+STRESS_MEMRATE_WRITE(1024, stress_uint64w1024_t)
+STRESS_MEMRATE_WRITE_RATE(1024, stress_uint64w1024_t)
+STRESS_MEMRATE_WRITE(512, stress_uint64w512_t)
+STRESS_MEMRATE_WRITE_RATE(512, stress_uint64w512_t)
+STRESS_MEMRATE_WRITE(256, stress_uint64w256_t)
+STRESS_MEMRATE_WRITE_RATE(256, stress_uint64w256_t)
+STRESS_MEMRATE_WRITE(128, stress_uint64w128_t)
+STRESS_MEMRATE_WRITE_RATE(128, stress_uint64w128_t)
 #endif
 #if defined(HAVE_INT128_T) && !defined(HAVE_VECMATH)
 STRESS_MEMRATE_WRITE(128, __uint128_t)
@@ -854,7 +867,7 @@ STRESS_MEMRATE_WRITE_RATE(16, uint16_t)
 STRESS_MEMRATE_WRITE(8, uint8_t)
 STRESS_MEMRATE_WRITE_RATE(8, uint8_t)
 
-static stress_memrate_info_t memrate_info[] = {
+static const stress_memrate_info_t memrate_info[] = {
 	{ "all",	MR_RW,  NULL,				NULL },
 #if defined(HAVE_ASM_X86_REP_STOSQ) &&	\
     !defined(__ILP32__)
@@ -862,16 +875,20 @@ static stress_memrate_info_t memrate_info[] = {
 #endif
 #if defined(HAVE_ASM_X86_REP_STOSD) &&	\
     !defined(__ILP32__)
-	{ "write32stow",MR_WR,	stress_memrate_write_stos32,	stress_memrate_write_stos_rate32 },
+	{ "write32stod",MR_WR,	stress_memrate_write_stos32,	stress_memrate_write_stos_rate32 },
 #endif
 #if defined(HAVE_ASM_X86_REP_STOSW) &&	\
     !defined(__ILP32__)
-	{ "write16stod",MR_WR,	stress_memrate_write_stos16,	stress_memrate_write_stos_rate16 },
+	{ "write16stow",MR_WR,	stress_memrate_write_stos16,	stress_memrate_write_stos_rate16 },
 #endif
 #if defined(HAVE_ASM_X86_REP_STOSB) &&	\
     !defined(__ILP32__)
 	{ "write8stob",	MR_WR,	stress_memrate_write_stos8,	stress_memrate_write_stos_rate8 },
 #endif
+#if defined(HAVE_ASM_X86_MOVDIRI) && 	\
+    defined(STRESS_ARCH_X86_64)
+	{ "write64ds",	MR_WR, stress_memrate_write_ds64,	stress_memrate_write_ds_rate64 },
+#endif
 #if defined(HAVE_NT_STORE128)
 	{ "write128nt",	MR_WR, stress_memrate_write_nt128,	stress_memrate_write_nt_rate128 },
 #endif
@@ -924,7 +941,7 @@ static void OPTIMIZE3 stress_memrate_init_data(
 {
 	register volatile uint32_t *ptr;
 
-	for (ptr = start; ptr < (uint32_t *)end; ptr++)
+	for (ptr = (uint32_t *)start; ptr < (uint32_t *)end; ptr++)
 		*ptr = stress_mwc32();
 }
 
@@ -941,8 +958,9 @@ static inline void *stress_memrate_mmap(stress_args_t *args, uint64_t sz)
 		MAP_ANONYMOUS, -1, 0);
 	/* Coverity Scan believes NULL can be returned, doh */
 	if (!ptr || (ptr == MAP_FAILED)) {
-		pr_err("%s: cannot allocate %" PRIu64 " K\n",
-			args->name, sz / 1024);
+		pr_err("%s: failed to mmap %" PRIu64 " K%s, errno=%d (%s)\n",
+			args->name, sz / 1024, stress_get_memfree_str(),
+			errno, strerror(errno));
 		ptr = MAP_FAILED;
 	} else {
 #if defined(HAVE_MADVISE) &&	\
@@ -1009,11 +1027,12 @@ static int stress_memrate_child(stress_args_t *args, void *ctxt)
 	context->start = buffer;
 	context->end = buffer_end;
 
+#if defined(HAVE_SIGLONGJMP)
 	if (sigsetjmp(jmpbuf, 1) != 0)
 		goto tidy;
-
 	if (stress_sighandler(args->name, SIGALRM, stress_memrate_alarm_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
+#endif
 
 	do {
 		if (context->memrate_method == 0) {
@@ -1030,8 +1049,10 @@ static int stress_memrate_child(stress_args_t *args, void *ctxt)
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
+#if defined(HAVE_SIGLONGJMP)
 tidy:
 	do_jmp = false;
+#endif
 	(void)munmap((void *)buffer, context->memrate_bytes);
 	return EXIT_SUCCESS;
 }
@@ -1042,16 +1063,17 @@ tidy:
  */
 static int stress_memrate(stress_args_t *args)
 {
-	int rc;
+	int rc, flag;
 	size_t i, stats_size;
 	stress_memrate_context_t context;
+	double inverse_n, geomean, rd_mantissa, rd_n, wr_mantissa, wr_n;
+	int64_t rd_exponent, wr_exponent;
 
 	context.memrate_bytes = DEFAULT_MEMRATE_BYTES;
 	context.memrate_rd_mbs = ~0ULL;
 	context.memrate_wr_mbs = ~0ULL;
 	context.memrate_flush = false;
 	context.memrate_method = 0; 	/* all */
-	int flag;
 
 	(void)stress_get_setting("memrate-bytes", &context.memrate_bytes);
 	(void)stress_get_setting("memrate-flush", &context.memrate_flush);
@@ -1066,7 +1088,7 @@ static int stress_memrate(stress_args_t *args)
 	flag = ((context.memrate_rd_mbs == 0) ? 0 : MR_RD) |
 	       ((context.memrate_wr_mbs == 0) ? 0 : MR_WR);
 	if ((flag & memrate_info[context.memrate_method].rdwr) == 0) {
-		pr_fail("%s: cannot use zero MB rate and just methood %s\n",
+		pr_fail("%s: cannot use zero MB rate and just method %s\n",
 			args->name, memrate_info[context.memrate_method].name);
 		return EXIT_FAILURE;
 	}
@@ -1077,8 +1099,10 @@ static int stress_memrate(stress_args_t *args)
 	context.stats = (stress_memrate_stats_t *)stress_mmap_populate(NULL, stats_size,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (context.stats == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zd byte statistics buffer, skipping stressor\n",
-			args->name, stats_size);
+		pr_inf_skip("%s: failed to mmap %zu byte statistics buffer%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, stats_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	for (i = 0; i < memrate_items; i++) {
@@ -1088,15 +1112,13 @@ static int stress_memrate(stress_args_t *args)
 	}
 
 	context.memrate_bytes = (context.memrate_bytes + 1023) & ~(1023ULL);
-	if (args->instance == 0) {
-		pr_inf("%s: using buffer size of %" PRIu64 "K, cache flushing %s\n", args->name,
-			context.memrate_bytes >> 10,
-			context.memrate_flush ? "enabled" : "disabled");
-		if ((context.memrate_bytes > MB) && (context.memrate_bytes & MB)) {
+	if (stress_instance_zero(args)) {
+		stress_usage_bytes(args, context.memrate_bytes, context.memrate_bytes);
+		pr_inf("%s: cache flushing %s\n", args->name,
+			context.memrate_flush ? "enabled" :
+			"disabled, cache flushing can be enabled with --memrate-flush option");
+		if ((context.memrate_bytes > MB) && (context.memrate_bytes & MB))
 			pr_inf("%s: for optimal speed, use multiples of 1 MB for --memrate-bytes\n", args->name);
-		}
-		if (!context.memrate_flush)
-			pr_inf("%s: cache flushing can be enabled with --memrate-flush option\n", args->name);
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
@@ -1107,22 +1129,64 @@ static int stress_memrate(stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	pr_block_begin();
+	rd_mantissa = 1.0;
+	rd_exponent = 0;
+	rd_n = 0.0;
+	wr_mantissa = 1.0;
+	wr_exponent = 0;
+	wr_n = 0.0;
+
 	for (i = 1; i < memrate_items; i++) {
 		if (!context.stats[i].valid)
 			continue;
 		if (context.stats[i].duration > 0.0) {
 			char tmp[32];
 			const double rate = context.stats[i].kbytes / (context.stats[i].duration * KB);
+			int e;
+			double f;
+
+			switch (memrate_info[i].rdwr) {
+			case MR_RD:
+				f = frexp((double)rate, &e);
+				rd_mantissa *= f;
+				rd_exponent += e;
+				rd_n += 1.0;
+				break;
+			case MR_WR:
+				f = frexp((double)rate, &e);
+				wr_mantissa *= f;
+				wr_exponent += e;
+				wr_n += 1.0;
+				break;
+			default:
+				break;
+			}
 
 			(void)snprintf(tmp, sizeof(tmp), "%s MB per sec", memrate_info[i].name);
 			stress_metrics_set(args, i, tmp,
 				rate, STRESS_METRIC_HARMONIC_MEAN);
+
 		} else {
 			pr_inf("%s: %10.10s: interrupted early\n",
 				args->name, memrate_info[i].name);
 		}
 	}
+
+	pr_block_begin();
+	if (rd_n > 0.0) {
+		inverse_n = 1.0 / rd_n;
+		geomean = pow(rd_mantissa, inverse_n) *
+			  pow(2.0, (double)rd_exponent * inverse_n);
+		pr_inf("%s: read rate %.2f MB per sec (geometric mean of per stressor read rates)\n",
+			args->name, geomean);
+	}
+	if (wr_n > 0.0) {
+		inverse_n = 1.0 / wr_n;
+		geomean = pow(wr_mantissa, inverse_n) *
+			  pow(2.0, (double)wr_exponent * inverse_n);
+		pr_inf("%s: write rate %.2f MB per sec (geometric mean of per stressor write rates)\n",
+			args->name, geomean);
+	}
 	pr_block_end();
 
 	(void)munmap((void *)context.stats, stats_size);
@@ -1141,13 +1205,13 @@ static const stress_opt_t opts[] = {
 	{ OPT_memrate_flush,  "memrate-flush",  TYPE_ID_BOOL, 0, 1, NULL },
 	{ OPT_memrate_rd_mbs, "memrate-rd-mbs", TYPE_ID_UINT64, 0, 1000000, NULL },
 	{ OPT_memrate_wr_mbs, "memrate-wr-mbs", TYPE_ID_UINT64, 0, 1000000, NULL },
-	{ OPT_memrate_method, "memrate-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_memmap_method },
+	{ OPT_memrate_method, "memrate-method", TYPE_ID_SIZE_T_METHOD, 0, 0, (void *)stress_memmap_method },
 	END_OPT,
 };
 
 const stressor_info_t stress_memrate_info = {
 	.stressor = stress_memrate,
-	.class = CLASS_MEMORY,
+	.classifier = CLASS_MEMORY,
 	.opts = opts,
 	.help = help
 };
