@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -104,6 +104,8 @@ static int do_mlock(
 	} else {
 		double t;
 
+		metrics_count = 0;
+
 		/* slower metrics timed mlock operations */
 		if (use_mlock2) {
 			const uint8_t rnd = stress_mwc8() >> 5;
@@ -154,6 +156,8 @@ static inline int do_mlock(
 	} else {
 		double t;
 
+		metrics_count = 0;
+
 		/* slower metrics timed mlock operations */
 		t = stress_time_now();
 		ret = shim_mlock((const void *)addr, len);
@@ -335,7 +339,7 @@ static int stress_mlock_child(stress_args_t *args, void *context)
 	 *  Under memory pressure we may need to scale back our
 	 *  mappings array to the point where it can fit into memory.
 	 */
-	mappings = MAP_FAILED;
+	mappings = (uint8_t **)MAP_FAILED;
 	while (stress_continue(args) && (mappings_len >= page_size)) {
 		mappings = (uint8_t **)mmap(NULL, mappings_len, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
@@ -460,7 +464,7 @@ static int stress_mlock_child(stress_args_t *args, void *context)
 				VOID_RET(int, shim_munlock((void *)((uint8_t *)addr + page_size), 0));
 			}
 			(void)stress_munmap_force((void *)addr, page_size * 3);
-			mappings[i] = MAP_FAILED;
+			mappings[i] = (uint8_t *)MAP_FAILED;
 		}
 
 		for (n = 0; n < max; n++) {
