@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -191,6 +191,8 @@ static void OPTIMIZE3 stress_parse_payload(
 			}
 			*nivcsw = (uint64_t)t->nivcsw;
 			break;
+		default:
+			break;
 		}
 		len += new_len;
 		na = (struct nlattr *)(uintptr_t)((char *)na + new_len);
@@ -244,7 +246,7 @@ static int OPTIMIZE3 stress_netlink_taskstats_monitor(
 				break;
 			if (na->nla_type == TASKSTATS_TYPE_AGGR_PID)
 				stress_parse_payload(args, na, pid, nivcsw);
-			na = (struct nlattr *)((char *)na + NLA_ALIGN(na->nla_len));
+			na = (struct nlattr *)(void *)((char *)na + NLA_ALIGN(na->nla_len));
 		}
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
