@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2025 Colin Ian King
+ * Copyright (C) 2021-2026 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -174,11 +174,24 @@
 #endif
 #endif
 
+#if defined(HAVE_SYS_TREE_H)
+#if defined(__QNX__) && __QNX__ >= 800
+/* QNX 8 contains a version of sys/tree.h which uses __uintptr_t and __unused */
+#ifndef __uintptr_t
+#define __uintptr_t uintptr_t
+#endif
+#ifndef __unused
+#define __unused
+#endif
+#endif
+#endif
+
 /*
  *  Per stressor misc rate metrics
  */
 #define STRESS_MISC_METRICS_MAX			(96)
 
+/* Process tracking info via pid */
 typedef struct stress_pid {
 	struct stress_pid *next;	/* next stress_pid in list, or NULL */
 	pid_t pid;			/* PID of process */
@@ -187,6 +200,7 @@ typedef struct stress_pid {
 	bool reaped;			/* successfully waited for */
 } stress_pid_t;
 
+/* Bogo-op counter state */
 typedef struct {
 	uint64_t counter;		/* bogo-op counter */
 	bool counter_ready;		/* ready flag */
@@ -195,28 +209,32 @@ typedef struct {
 	bool padding;			/* padding */
 } stress_counter_info_t;
 
+/* Shared mmap'd pages */
 typedef struct {
 	void *page_none;		/* mmap'd PROT_NONE page */
 	void *page_ro;			/* mmap'd PROT_RO page */
 	void *page_wo;			/* mmap'd PROT_WO page */
 } stress_mapped_t;
 
+/* Individual metric */
 typedef struct {
 	char *description;		/* description of metric */
 	double value;			/* value of metric */
 	int mean_type;			/* type of metric, geometric or harmonic mean */
 } stress_metrics_item_t;
 
+/* Per stressor metrics */
 typedef struct {
 	size_t max_metrics;
 	stress_metrics_item_t items[STRESS_MISC_METRICS_MAX];
 } stress_metrics_data_t;
 
-/* stressor args */
+/* Run-time stressor args */
 typedef struct {
 	struct {
-		uint64_t max_ops;		/* max number of bogo ops */
-		stress_counter_info_t ci;	/* counter info struct */
+		volatile uint64_t max_ops; /* max number of bogo ops */
+		stress_counter_info_t ci;  /* counter info struct */
+		bool possibly_oom_killed;  /* was oom killed? */
 	} bogo;
 	const char *name;		/* stressor name */
 	uint32_t instance;		/* stressor instance # */
@@ -230,6 +248,7 @@ typedef struct {
 	const struct stressor_info *info; /* stressor info */
 } stress_args_t;
 
+/* Run-time stressor descriptor list */
 typedef struct stress_stressor_info {
 	struct stress_stressor_info *next; /* next proc info struct in list */
 	struct stress_stressor_info *prev; /* prev proc info struct in list */
@@ -253,8 +272,11 @@ typedef struct stress_stressor_info {
 #include "core-parse-opts.h"
 #include "core-perf.h"
 #include "core-setting.h"
+#include "core-signal.h"
+#include "core-stack.h"
 #include "core-log.h"
 #include "core-lock.h"
+#include "core-memory.h"
 #include "core-mwc.h"
 #include "core-rapl.h"
 #include "core-sched.h"
@@ -440,7 +462,7 @@ typedef enum {
 	VERIFY_ALWAYS   = 0x02,		/* verification always enabled */
 } stress_verify_t;
 
-/* stressor information */
+/* Per stressor information, as defined in every stressor source */
 typedef struct stressor_info {
 	int (*stressor)(stress_args_t *args);	/* stressor function */
 	int (*supported)(const char *name);	/* return 0 = supported, -1, not */
@@ -450,7 +472,7 @@ typedef struct stressor_info {
 	void (*set_limit)(uint64_t max);/* set limits */
 	const stress_opt_t *opts;	/* new option settings */
 	const stress_help_t *help;	/* stressor help options */
-	const stress_class_t class;	/* stressor class */
+	const stress_class_t classifier;/* stressor class */
 	const stress_verify_t verify;	/* verification mode */
 	const char *unimplemented_reason;	/* unsupported reason message */
 } stressor_info_t;
@@ -535,21 +557,23 @@ extern const char stress_config[];
 /* Generic bit setting on an array macros */
 #define STRESS_NBITS(a)		(sizeof(a[0]) * 8)
 #define STRESS_GETBIT(a, i)	(a[i / STRESS_NBITS(a)] & \
-				 (TYPEOF_CAST(a[0])1U << (i & (STRESS_NBITS(a)-1))))
+				 (TYPEOF_CAST(a[0])1U << ((size_t)i & (STRESS_NBITS(a)-1))))
 #define STRESS_CLRBIT(a, i)	(a[i / STRESS_NBITS(a)] &= \
-				 ~(TYPEOF_CAST(a[0])1U << (i & (STRESS_NBITS(a)-1))))
+				 ~(TYPEOF_CAST(a[0])1U << ((size_t)i & (STRESS_NBITS(a)-1))))
 #define STRESS_SETBIT(a, i)	(a[i / STRESS_NBITS(a)] |= \
-				 (TYPEOF_CAST(a[0])1U << (i & (STRESS_NBITS(a)-1))))
+				 (TYPEOF_CAST(a[0])1U << ((size_t)i & (STRESS_NBITS(a)-1))))
 
 #define SIZEOF_ARRAY(a)		(sizeof(a) / sizeof(a[0]))
 
+/* Per interrupt stat counters */
 typedef struct {
-	uint64_t count_start;
-	uint64_t count_stop;
+	uint64_t count_start;		/* interrupt count at start */
+	uint64_t count_stop;		/* interrupt count at end */
 } stress_interrupts_t;
 
+/* C-state stats */
 typedef struct {
-	bool   valid;
+	bool   valid;			/* C state is valid? */
 	double time[STRESS_CSTATES_MAX];
 	double residency[STRESS_CSTATES_MAX];
 } stress_cstate_stats_t;
@@ -585,6 +609,7 @@ typedef struct stress_stats {
 	long int rusage_maxrss;		/* rusage max RSS, 0 = unused */
 } stress_stats_t;
 
+/* Shared heap info */
 typedef struct shared_heap {
 	void *str_list_head;		/* list of heap strings */
 	void *lock;			/* heap global lock */
@@ -642,8 +667,8 @@ typedef struct {
 	} atomic ALIGN64;		/* Shared atomic temp vars */
 	struct {
 		void *lock;
-		int64_t row;
-	} fractal;
+		int32_t row;
+	} fractal;			/* Fractal stressor row state */
 	struct {
 		/* futexes must be aligned to avoid -EINVAL */
 		uint32_t futex[STRESS_PROCS_MAX] ALIGNED(4);/* Shared futexes */
@@ -700,10 +725,11 @@ typedef struct stress {
 extern stress_stressor_t *g_stressor_current;
 
 /* Various global option settings and flags */
-extern const char g_app_name[];		/* Name of application */
+extern const char g_prog_name[];	/* Name of programme */
 extern stress_shared_t *g_shared;	/* shared memory */
 extern uint64_t	g_opt_timeout;		/* timeout in seconds */
 extern uint64_t	g_opt_flags;		/* option flags */
+extern uint32_t g_pr_log_flags;		/* pr_* logging flags */
 extern volatile bool g_stress_continue_flag; /* false to exit stressor */
 extern jmp_buf g_error_env;		/* parsing error env */
 extern void *g_nowt;			/* void pointer to NULL */
@@ -821,7 +847,7 @@ static inline void ALWAYS_INLINE stress_force_killed_bogo(stress_args_t *args)
  *	add val to the stressor bogo ops counter with lock, return true
  *	if stress_continue is true
  */
-static inline void stress_bogo_add_lock(stress_args_t *args, void *lock, const int64_t val)
+static inline void stress_bogo_add_lock(stress_args_t *args, void *lock, const uint64_t val)
 {
 	/*
 	 *  Failure in lock acquire, don't bump counter
@@ -858,7 +884,17 @@ static inline bool stress_bogo_inc_lock(stress_args_t *args, void *lock, const b
 	return ret;
 }
 
+/*
+ *  stress_instance_zero()
+ *	return true if stressor is the 0th instance of N stressor instances
+ */
+static inline bool ALWAYS_INLINE stress_instance_zero(stress_args_t *args)
+{
+	return args->instance == 0;
+}
+
 #include "core-helper.h"
+#include "core-filesystem.h"
 
 #define STRESS_METRIC_GEOMETRIC_MEAN	(0x1)
 #define STRESS_METRIC_HARMONIC_MEAN	(0x2)
@@ -871,6 +907,7 @@ extern void stress_shared_unmap(void);
 extern void stress_log_system_mem_info(void);
 extern void stress_metrics_set_const_check(stress_args_t *args,
 	const size_t idx, char *description, const bool const_description, const double value, const int mean_type);
+extern WARN_UNUSED ssize_t stress_stressor_find(const char *name);
 #if defined(HAVE_BUILTIN_CONSTANT_P)
 #define stress_metrics_set(args, idx, description, value, mean_type)	\
 	stress_metrics_set_const_check(args, idx, description, __builtin_constant_p(description), value, mean_type)
@@ -885,6 +922,9 @@ extern void stress_metrics_set_const_check(stress_args_t *args,
 int unlink(const char *pathname) __attribute__((deprecated("use shim_unlink")));
 int unlinkat(int dirfd, const char *pathname, int flags) __attribute__((deprecated("use shim_unlinkat")));
 int rmdir(const char *pathname) __attribute__((deprecated("use shim_rmdir")));
+size_t strlcpy(char *dst, const char *src, size_t size) __attribute__((deprecated("shim_strscpy")));
+size_t strlcat(char *dst, const char *src, size_t size) __attribute__((deprecated("shim_strlcat")));
+
 #endif
 
 #endif
