@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,7 +34,7 @@ typedef int (*stress_open_func_t)(stress_args_t *args, const char *temp_dir, con
 static const stress_help_t help[] = {
 	{ "o N", "open N",		"start N workers exercising open/close" },
 	{ NULL, "open-fd",		"open files in /proc/$pid/fd" },
-	{ NULL,	"open-max N",		"specficify maximum number of files to open" },
+	{ NULL,	"open-max N",		"specify maximum number of files to open" },
 	{ NULL,	"open-ops N",		"stop after N open/close bogo operations" },
 	{ NULL,	NULL,			NULL }
 };
@@ -42,7 +42,7 @@ static const stress_help_t help[] = {
 static size_t open_count;
 static int *open_perms;
 
-static int open_flags[] = {
+static const int open_flags[] = {
 #if defined(O_APPEND)
 	O_APPEND,
 #endif
@@ -182,13 +182,13 @@ static void stress_open_max(const char *opt_name, const char *opt_arg, stress_ty
 	(void)opt_name;
 
 	*type_id = TYPE_ID_SIZE_T;
-	*open_max = (size_t)stress_get_uint64_percent(opt_arg, 1, (uint64_t)max_fds,
+	*open_max = (size_t)stress_get_uint64_percent(opt_arg, 1, (uint64_t)max_fds, NULL,
                         "cannot determine maximum number of file descriptors");
 }
 
 static const stress_opt_t opts[] = {
 	{ OPT_open_fd,  "open-fd",  TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_open_max,	"open-max", TYPE_ID_CALLBACK, 0, 1, stress_open_max },
+	{ OPT_open_max,	"open-max", TYPE_ID_CALLBACK, 0, 1, (void *)stress_open_max },
 	END_OPT,
 };
 
@@ -212,7 +212,7 @@ static int obsolete_futimesat(
 	if (LIKELY((ret == 0) || (errno != ENOSYS)))
 		return ret;
 #endif
-#if defined(HAVE_FUTIMESAT)
+#if defined(HAVE_FUTIMESAT_DEPRECATED)
 	/* Try libc variant next */
 	ret = (int)futimesat(dir_fd, pathname, tv);
 	if (LIKELY((ret == 0) || (errno != ENOSYS)))
@@ -398,7 +398,7 @@ static int open_dev_zero_rd(
 	int flags = 0;
 
 	(void)args;
-	(void)temp_dir,
+	(void)temp_dir;
 	(void)pid;
 
 #if defined(O_ASYNC)
@@ -522,7 +522,7 @@ static int open_tmpfile_no_rdwr(
 
 #if defined(HAVE_POSIX_OPENPT) &&	\
     defined(O_RDWR) &&			\
-    defined(N_NOCTTY)
+    defined(O_NOCTTY)
 static int open_pt(
 	stress_args_t *args,
 	const char *temp_dir,
@@ -530,11 +530,20 @@ static int open_pt(
 	double *duration,
 	double *count)
 {
+	int fd;
+	double t;
+
 	(void)args;
 	(void)temp_dir;
 	(void)pid;
 
-	return posix_openpt(O_RDWR | O_NOCTTY);
+	t = stress_time_now();
+	fd = posix_openpt(O_RDWR | O_NOCTTY);
+	if (LIKELY(fd >= 0)) {
+		(*duration) += stress_time_now() - t;
+		(*count) += 1.0;
+	}
+	return fd;
 }
 #endif
 
@@ -896,23 +905,24 @@ static int open_rdonly_trunc(
 	double *count)
 {
 	char filename[PATH_MAX];
-	int fd;
+	int fd1, fd2;
 
 	(void)args;
 
 	(void)snprintf(filename, sizeof(filename), "%s/stress-open-%" PRIdMAX "-%" PRIu32,
 		temp_dir, (intmax_t)pid, stress_mwc32());
 
-	fd = open_arg3(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, duration, count);
-	if (fd < 0)
-		return fd;
+	fd1 = open_arg3(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, duration, count);
+	if (fd1 < 0)
+		return fd1;
 
 	/* undefined behaviour, will open, may truncate on some systems */
-	fd = open_arg2(filename, O_RDONLY | O_TRUNC, duration, count);
-	(void)fd;
+	fd2 = open_arg2(filename, O_RDONLY | O_TRUNC, duration, count);
+	if (fd2 >= 0)
+		(void)close(fd2);
 	(void)shim_unlink(filename);
 
-	return fd;
+	return fd1;
 }
 #endif
 
@@ -955,7 +965,7 @@ static int open_modes(
 }
 #endif
 
-static stress_open_func_t open_funcs[] = {
+static const stress_open_func_t open_funcs[] = {
 #if defined(O_CREAT)
 	open_flag_perm,
 #endif
@@ -979,7 +989,7 @@ static stress_open_func_t open_funcs[] = {
 #endif
 #if defined(HAVE_POSIX_OPENPT) &&	\
     defined(O_RDWR) &&			\
-    defined(N_NOCTTY)
+    defined(O_NOCTTY)
 	open_pt,
 #endif
 #if defined(O_CREAT)
@@ -1109,12 +1119,15 @@ static int stress_open(stress_args_t *args)
 		fds = (int *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 		if (fds == MAP_FAILED) {
-			pr_inf_skip("%s: cannot mmap %zd file descriptors\n", args->name, open_max);
+			pr_inf_skip("%s: cannot mmap %zu file descriptors%s, "
+				"errno=%d (%s), skipping stressor\n",
+				args->name, open_max, stress_get_memfree_str(),
+				errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
 		}
 	}
 	if (!args->instance)
-		pr_inf("%s: using a maximum of %zd file descriptors\n", args->name, open_max);
+		pr_inf("%s: using a maximum of %zu file descriptors\n", args->name, open_max);
 	stress_set_vma_anon_name(fds, sz, "fds");
 
 	if (open_fd) {
@@ -1124,6 +1137,8 @@ static int stress_open(stress_args_t *args)
 			pid = fork();
 
 			if (pid == 0) {
+				stress_set_proc_state(args->name, STRESS_STATE_RUN);
+				stress_set_make_it_fail();
 				stress_fd_dir(path, &duration, &count);
 				_exit(0);
 			}
@@ -1176,7 +1191,7 @@ static int stress_open(stress_args_t *args)
 			stress_read_fdinfo(mypid, fds[i]);
 
 			if (UNLIKELY((i & 8191) == 8191))
-				sync();
+				shim_sync();
 
 			stress_bogo_inc(args);
 		}
@@ -1187,7 +1202,7 @@ close_all:
 		 *  try fast close of a range, fall back to
 		 *  normal close if ENOSYS
 		 */
-		sync();
+		shim_sync();
 		errno = 0;
 		ret = shim_close_range(min_fd, max_fd, 0);
 		if (ret < 0) {
@@ -1215,7 +1230,7 @@ close_all:
 
 const stressor_info_t stress_open_info = {
 	.stressor = stress_open,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.help = help
 };
