@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-affinity.h"
 #include "core-builtin.h"
+#include "core-mmap.h"
 
 #include <sys/ioctl.h>
 
@@ -92,7 +93,7 @@ static void pipe_change_size(
 	} else {
 		if (UNLIKELY((size_t)sz != pipe_size)) {
 			pr_err("%s: cannot set desired pipe size, "
-				"pipe %zd, defaulting to size %zd\n",
+				"pipe %zu, defaulting to size %zd\n",
 				args->name, pipe_size, sz);
 		}
 	}
@@ -163,7 +164,7 @@ static int stress_pipe_read_generic_verify(
 #if defined(FIONREAD)
 	register int i = 0;
 #endif
-	register const uint32_t *const buf32 = (uint32_t *)buf;
+	register const uint32_t *const buf32 = (uint32_t *)shim_assume_aligned(buf, 4);
 
 	while (stress_continue_flag()) {
 		register ssize_t n;
@@ -289,13 +290,15 @@ static int stress_pipe_read_splice_verify(
 	const size_t nbufs = buf_size / pipe_data_size;
 	const size_t offset_end = nbufs * pipe_data_size;
 
+	buf = (char *)shim_assume_aligned(buf, 4);
+
 	iov.iov_len = pipe_data_size;
 
 	while (stress_continue_flag()) {
 		register ssize_t n;
 
 		iov.iov_base = buf + offset;
-		buf32 = (uint32_t *)iov.iov_base;
+		buf32 = (uint32_t *)shim_assume_aligned(iov.iov_base, 4);
 		offset += pipe_data_size;
 		if (UNLIKELY(offset >= offset_end))
 			offset = 0;
@@ -390,7 +393,7 @@ static int stress_pipe_write_generic_verify(
 	uint32_t val)
 {
 	int rc = 0;
-	register uint32_t *const buf32 = (uint32_t *)buf;
+	register uint32_t *const buf32 = (uint32_t *)shim_assume_aligned(buf, 4);
 	register uint64_t bytes = 0;
 
 	do {
@@ -461,9 +464,10 @@ static int stress_pipe_write_splice(
 				break;
 			}
 			continue;
+		} else {
+			stress_bogo_inc(args);
+			bytes += ret;
 		}
-		stress_bogo_inc(args);
-		bytes += pipe_data_size;
 	} while (stress_continue(args));
 
 	*pbytes64 = bytes;
@@ -490,6 +494,7 @@ static int stress_pipe_write_splice_verify(
 	const size_t nbufs = buf_size / pipe_data_size;
 	const size_t offset_end = nbufs * pipe_data_size;
 
+	buf = (char *)shim_assume_aligned(buf, 4);
 	iov.iov_len = pipe_data_size;
 
 	do {
@@ -575,7 +580,7 @@ static int stress_pipe(stress_args_t *args)
 	if (pipe_vmsplice) {
 		pipe_vmsplice = false;
 		if (stress_instance_zero(args))
-			pr_inf("%s: no pipe packet mode with O_DIRECT, disabled pipe vmsplicing\n", args->name);
+			pr_inf("%s: no pipe packet mode without O_DIRECT, disabled pipe vmsplicing\n", args->name);
 	}
 #endif
 
@@ -660,6 +665,8 @@ again:
 	} else if (pid == 0) {
 		int ret;
 
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
+		stress_set_make_it_fail();
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 		(void)stress_change_cpu(args, parent_cpu);
