@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +41,7 @@ static int stress_pkey(stress_args_t *args)
 	const size_t page_size = args->page_size;
 	const size_t pages_size = page_size * PAGES_TO_EXERCISE;
 
-	pages = stress_mmap_populate(NULL, pages_size,
+	pages = (uint8_t *)stress_mmap_populate(NULL, pages_size,
 		PROT_READ | PROT_WRITE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (pages == MAP_FAILED) {
