@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -77,11 +77,12 @@ static int stress_pty(stress_args_t *args)
 			pty_max = MIN_PTYS;
 	}
 
-	ptys = (stress_pty_info_t *)calloc(pty_max, sizeof(*ptys));
+	ptys = (stress_pty_info_t *)calloc((size_t)pty_max, sizeof(*ptys));
 	if (!ptys) {
-		pr_inf_skip("%s: allocation of pty array failed, errno=%d (%s) "
+		pr_inf_skip("%s: allocation of %zu pty array failed%s, "
 			"skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, (size_t)pty_max,
+			stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -161,6 +162,7 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
+
 #if defined(HAVE_TCDRAIN)
 			{
 				if (UNLIKELY((tcdrain(ptys[i].follower) < 0) &&
@@ -171,6 +173,7 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
+
 #if defined(HAVE_TCFLUSH)
 			{
 #if defined(TCIFLUSH)
@@ -189,15 +192,16 @@ static int stress_pty(stress_args_t *args)
 #endif
 #if defined(TCIOFLUSH)
 				if (UNLIKELY(tcflush(ptys[i].follower, TCIOFLUSH) < 0)) {
-					pr_fail("%s: tcflush TCOOFLUSH on follower pty failed, errno=%d (%s)\n",
+					pr_fail("%s: tcflush TCIOFLUSH on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
 				}
 #endif
 			}
 #endif
-#if defined(HAVE_TCFLOW)
-#if defined(TCOOFF) && \
+
+#if defined(HAVE_TCFLOW) &&	\
+    defined(TCOOFF) && 		\
     defined(TCOON)
 			{
 				if (UNLIKELY(tcflow(ptys[i].follower, TCOOFF) < 0)) {
@@ -212,7 +216,9 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
-#if defined(TCIOFF) && \
+
+#if defined(HAVE_TCFLOW) &&	\
+    defined(TCIOFF) &&		\
     defined(TCION)
 			{
 				if (UNLIKELY(tcflow(ptys[i].follower, TCIOFF) < 0)) {
@@ -227,8 +233,9 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
-#endif
-#if defined(TCGETS)
+
+#if defined(TCGETS) &&	\
+    defined(TCSETS)
 			{
 				struct termios ios;
 
@@ -237,46 +244,61 @@ static int stress_pty(stress_args_t *args)
 					pr_fail("%s: ioctl TCGETS on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
+				} else {
+					if (UNLIKELY((ioctl(ptys[i].follower, TCSETS, &ios) < 0) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: ioctl TCSETS on follower pty failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
 				}
 			}
 #endif
-#if defined(TCSETS)
+
+#if defined(TCGETS) &&	\
+    defined(TCSETSW)
 			{
 				struct termios ios;
 
-				if (UNLIKELY((ioctl(ptys[i].follower, TCSETS, &ios) < 0) &&
+				if (UNLIKELY((ioctl(ptys[i].follower, TCGETS, &ios) < 0) &&
 					     (errno != EINTR))) {
-					pr_fail("%s: ioctl TCSETS on follower pty failed, errno=%d (%s)\n",
+					pr_fail("%s: ioctl TCGETS on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
+				} else {
+					if (UNLIKELY((ioctl(ptys[i].follower, TCSETSW, &ios) < 0) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: ioctl TCSETSW on follower pty failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
 				}
 			}
 #endif
-#if defined(TCSETSW)
+
+#if defined(TCGETS) &&	\
+    defined(TCSETSF)
 			{
 				struct termios ios;
 
-				if (UNLIKELY((ioctl(ptys[i].follower, TCSETSW, &ios) < 0) &&
+				if (UNLIKELY((ioctl(ptys[i].follower, TCGETS, &ios) < 0) &&
 					     (errno != EINTR))) {
-					pr_fail("%s: ioctl TCSETSW on follower pty failed, errno=%d (%s)\n",
+					pr_fail("%s: ioctl TCGETS on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
+				} else {
+					if (UNLIKELY((ioctl(ptys[i].follower, TCSETSF, &ios) < 0) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: ioctl TCSETSF on follower pty failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
 				}
 			}
 #endif
-#if defined(TCSETSF)
-			{
-				struct termios ios;
 
-				if (UNLIKELY((ioctl(ptys[i].follower, TCSETSF, &ios) < 0) &&
-					     (errno != EINTR))) {
-					pr_fail("%s: ioctl TCSETSF on follower pty failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
-					rc = EXIT_FAILURE;
-				}
-			}
-#endif
-#if defined(TCGETA)
+#if defined(TCGETA) &&	\
+    defined(TCSETA)
 			{
 				struct termio io;
 
@@ -285,45 +307,59 @@ static int stress_pty(stress_args_t *args)
 					pr_fail("%s: ioctl TCGETA on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
+				} else {
+					if (UNLIKELY((ioctl(ptys[i].follower, TCSETA, &io) < 0) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: ioctl TCSETA on follower pty failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
 				}
 			}
 #endif
-#if defined(TCSETA)
+
+#if defined(TCGETA) &&	\
+    defined(TCSETAW)
 			{
 				struct termio io;
 
-				if (UNLIKELY((ioctl(ptys[i].follower, TCSETA, &io) < 0) &&
+				if (UNLIKELY((ioctl(ptys[i].follower, TCGETA, &io) < 0) &&
 					     (errno != EINTR))) {
-					pr_fail("%s: ioctl TCSETA on follower pty failed, errno=%d (%s)\n",
+					pr_fail("%s: ioctl TCGETA on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
+				} else {
+					if (UNLIKELY((ioctl(ptys[i].follower, TCSETAW, &io) < 0) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: ioctl TCSETAW on follower pty failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
 				}
 			}
 #endif
-#if defined(TCSETAW)
+
+#if defined(TCGETA) &&	\
+    defined(TCSETAF)
 			{
 				struct termio io;
 
-				if (UNLIKELY((ioctl(ptys[i].follower, TCSETAW, &io) < 0) &&
+				if (UNLIKELY((ioctl(ptys[i].follower, TCGETA, &io) < 0) &&
 					     (errno != EINTR))) {
-					pr_fail("%s: ioctl TCSETAW on follower pty failed, errno=%d (%s)\n",
+					pr_fail("%s: ioctl TCGETA on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
+				} else {
+					if (UNLIKELY((ioctl(ptys[i].follower, TCSETAF, &io) < 0) &&
+						     (errno != EINTR))) {
+						pr_fail("%s: ioctl TCSETAF on follower pty failed, errno=%d (%s)\n",
+							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
 				}
 			}
 #endif
-#if defined(TCSETAF)
-			{
-				struct termio io;
 
-				if (UNLIKELY((ioctl(ptys[i].follower, TCSETAF, &io) < 0) &&
-					     (errno != EINTR))) {
-					pr_fail("%s: ioctl TCSETAF on follower pty failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
-					rc = EXIT_FAILURE;
-				}
-			}
-#endif
 #if defined(TIOCGLCKTRMIOS)
 			{
 				struct termios ios;
@@ -336,7 +372,9 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
-#if defined(TIOCGWINSZ)
+
+#if defined(TIOCGWINSZ) &&	\
+    defined(TIOCSWINSZ)
 			{
 				struct winsize ws;
 
@@ -346,12 +384,6 @@ static int stress_pty(stress_args_t *args)
 						args->name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
 				}
-			}
-#endif
-#if defined(TIOCSWINSZ)
-			{
-				struct winsize ws;
-
 				if (UNLIKELY((ioctl(ptys[i].follower, TIOCSWINSZ, &ws) < 0) &&
 					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TIOCSWINSZ on follower pty failed, errno=%d (%s)\n",
@@ -360,6 +392,7 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
+
 #if defined(FIONREAD)
 			{
 				int arg;
@@ -372,6 +405,7 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
+
 #if defined(TIOCINQ)
 			{
 				int arg;
@@ -384,6 +418,7 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
+
 #if defined(TIOCOUTQ)
 			{
 				int arg;
@@ -396,19 +431,20 @@ static int stress_pty(stress_args_t *args)
 				}
 			}
 #endif
-#if defined(TIOCGPTLCK)
+
+#if defined(TIOCGPTLCK) &&	\
+    defined(TIOCSPTLCK)
 			{
 				int ret, locked = 0;
 
 				ret = ioctl(ptys[i].leader, TIOCGPTLCK, &locked);
-#if defined(TIOCSPTLCK)
 				if (ret == 0)
 					ret = ioctl(ptys[i].leader, TIOCSPTLCK, &locked);
-#endif
 
 				(void)ret;
 			}
 #endif
+
 #if defined(TIOCGPTN)
 			{
 				unsigned int ptynum = 0;
@@ -416,73 +452,81 @@ static int stress_pty(stress_args_t *args)
 				(void)ioctl(ptys[i].leader, TIOCGPTN, &ptynum);	/* BSD */
 			}
 #endif
-#if defined(TIOCGPKT)
+
+#if defined(TIOCGPKT) &&	\
+    defined(TIOCPKT)
+
 			{
 				int val, ret;
 
 				ret = ioctl(ptys[i].leader, TIOCGPKT, &val);
-#if defined(TIOCPKT)
 				if (ret == 0)
 					ret = ioctl(ptys[i].leader, TIOCPKT, &val);
-#endif
 				(void)ret;
 			}
 #endif
+
+#if defined(HAVE_CFGETISPEED) &&	\
+    defined(HAVE_CFGETOSPEED)
 			{
 				struct termios ios;
 
 				if (tcgetattr(ptys[i].follower, &ios) == 0) {
-#if defined(HAVE_CFGETISPEED)
 					(void)cfgetispeed(&ios);
-#endif
-#if defined(HAVE_CFGETOSPEED)
 					(void)cfgetospeed(&ios);
-#endif
-				}
-				if (tcgetattr(ptys[i].leader, &ios) == 0) {
-#if defined(HAVE_CFGETISPEED)
-					(void)cfgetispeed(&ios);
-#endif
-#if defined(HAVE_CFGETOSPEED)
-					(void)cfgetospeed(&ios);
-#endif
 				}
 			}
+#endif
+
+#if defined(HAVE_CFGETISPEED) &&	\
+    defined(HAVE_CFGETOSPEED)
+			{
+				struct termios ios;
+
+				if (tcgetattr(ptys[i].leader, &ios) == 0) {
+					(void)cfgetispeed(&ios);
+					(void)cfgetospeed(&ios);
+				}
+			}
+#endif
+
 			if (UNLIKELY(!stress_continue_flag()))
 				goto clean;
-		}
+
 #if defined(TIOCSETD) &&	\
     defined(TIOCGETD) &&	\
     defined(TCXONC)
-		if ((args->instance == 0) && (fcntl(ptys[i].follower, F_SETFL, O_NONBLOCK) == 0)) {
+			if (stress_instance_zero(args) &&
+			   (fcntl(ptys[i].follower, F_SETFL, O_NONBLOCK) == 0)) {
 #if defined(NR_LDISCS)
-			const int max_ldisc = NR_LDISCS;
+				const int max_ldisc = NR_LDISCS;
 #else
-			const int max_ldisc = 32;
+				const int max_ldisc = 32;
 #endif
-			int ldisc, orig_ldisc;
+				int ldisc, orig_ldisc;
 
-			if (ioctl(ptys[i].follower, TIOCGETD, &orig_ldisc) == 0) {
-				pr_block_begin();
-				for (ldisc = 0; ldisc < max_ldisc; ldisc++) {
-					int j;
+				if (ioctl(ptys[i].follower, TIOCGETD, &orig_ldisc) == 0) {
+					pr_block_begin();
+					for (ldisc = 0; ldisc < max_ldisc; ldisc++) {
+						int j;
 
-					if (ioctl(ptys[i].follower, TIOCSETD, &ldisc) < 0)
-						break;
-					for (j = 0; j < 256; j++) {
-						if (ioctl(ptys[i].follower, TCXONC, 0) < 0)
-							break;
-						VOID_RET(ssize_t, write(ptys[i].follower, "", 1));
-						if (ioctl(ptys[i].follower, TCXONC, 1) < 0)
+						if (ioctl(ptys[i].follower, TIOCSETD, &ldisc) < 0)
 							break;
+						for (j = 0; j < 256; j++) {
+							if (ioctl(ptys[i].follower, TCXONC, 0) < 0)
+								break;
+							VOID_RET(ssize_t, write(ptys[i].follower, "", 1));
+							if (ioctl(ptys[i].follower, TCXONC, 1) < 0)
+								break;
+						}
 					}
+					VOID_RET(int, ioctl(ptys[i].follower, TIOCSETD, &orig_ldisc));
+					pr_block_end();
+					(void)shim_sched_yield();
 				}
-				VOID_RET(int, ioctl(ptys[i].follower, TIOCSETD, &orig_ldisc));
-				pr_block_end();
-				(void)shim_sched_yield();
 			}
-		}
 #endif
+		}
 
 #if defined(HAVE_PATHCONF)
 #if defined(_PC_MAX_CANON)
@@ -518,7 +562,7 @@ clean:
 
 const stressor_info_t stress_pty_info = {
 	.stressor = stress_pty,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -526,7 +570,7 @@ const stressor_info_t stress_pty_info = {
 #else
 const stressor_info_t stress_pty_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
