@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -189,7 +189,7 @@ static int stress_ramfs_fs_ops(
 			off_t offset = 0, scale;
 
 			for (scale = 80; scale <= 100; scale++) {
-				off_t end = scale * (ramfs_size / 100);
+				const off_t end = scale * (ramfs_size / 100);
 
 				errno = 0;
 				if (shim_fallocate(fd, 0, offset, end - offset) < 0)
@@ -197,12 +197,13 @@ static int stress_ramfs_fs_ops(
 				offset = end;
 			}
 			if (g_opt_flags & OPT_FLAGS_AGGRESSIVE) {
-				uint64_t i;
 				uint8_t buf[8192] ALIGN64;
 				const size_t sz = sizeof(buf);
 
 				stress_uint8rnd4(buf, sz);
 				if (lseek(fd, 0, SEEK_SET) == 0) {
+					uint64_t i;
+
 					for (i = 0; i < ramfs_size; i += sz) {
 						const uint64_t r = stress_mwc64();
 
@@ -299,13 +300,13 @@ static int stress_ramfs_child(stress_args_t *args)
 
 	if (ramfs_size & (page_size - 1)) {
 		ramfs_size &= page_mask;
-		pr_inf("ramfs: rounding ramfs-size to %" PRIu64 " x %" PRId64 "K pages\n",
+		pr_inf("ramfs: rounding ramfs-size to %" PRIu64 " x %" PRIu64 "K pages\n",
 			ramfs_size / page_size, page_size >> 10);
 	}
 
 	stress_temp_dir(pathname, sizeof(pathname), args->name,
 		args->pid, args->instance);
-	if (mkdir(pathname, S_IRGRP | S_IWGRP) < 0) {
+	if (mkdir(pathname, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP) < 0) {
 		pr_fail("%s: cannot mkdir %s, errno=%d (%s)\n",
 			args->name, pathname, errno, strerror(errno));
 		return EXIT_FAILURE;
@@ -422,8 +423,7 @@ skip_fsopen:
 
 #endif
 		stress_bogo_inc(args);
-	} while (keep_mounting && stress_continue(args) &&
-		 (!args->bogo.max_ops || (stress_bogo_get(args) < args->bogo.max_ops)));
+	} while (keep_mounting && stress_continue(args));
 
 cleanup:
 	stress_ramfs_umount(args, realpathname);
@@ -471,14 +471,14 @@ again:
 				}
 				(void)shim_waitpid(pid, &status, 0);
 			} else if (WIFSIGNALED(status)) {
-				pr_dbg("%s: child died: %s (instance %d)\n",
+				pr_dbg("%s: child died: %s (instance %" PRIu32 ")\n",
 					args->name, stress_strsignal(WTERMSIG(status)),
 					args->instance);
 				/* If we got killed by OOM killer, re-start */
 				if (WTERMSIG(status) == SIGKILL) {
 					stress_log_system_mem_info();
 					pr_dbg("%s: assuming killed by OOM killer, "
-						"restarting again (instance %d)\n",
+						"restarting again (instance %" PRIu32 ")\n",
 						args->name, args->instance);
 					goto again;
 				}
@@ -489,6 +489,8 @@ again:
 				return EXIT_NO_RESOURCE;
 			}
 		} else {
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
+			stress_set_make_it_fail();
 			_exit(stress_ramfs_child(args));
 		}
 	} while (stress_continue(args));
@@ -501,7 +503,7 @@ finish:
 
 const stressor_info_t stress_ramfs_info = {
 	.stressor = stress_ramfs_mount,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.supported = stress_ramfs_supported,
 	.verify = VERIFY_ALWAYS,
@@ -510,7 +512,7 @@ const stressor_info_t stress_ramfs_info = {
 #else
 const stressor_info_t stress_ramfs_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.supported = stress_ramfs_supported,
 	.verify = VERIFY_ALWAYS,
