@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -338,7 +338,7 @@ static int stress_revio(stress_args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return rc;
 	}
-	buf = alloc_buf;
+	buf = (uint8_t *)alloc_buf;
 #else
 	/* Work around lack of posix_memalign */
 	alloc_buf = malloc((size_t)DEFAULT_REVIO_WRITE_SIZE + BUF_ALIGNMENT);
@@ -481,7 +481,7 @@ finish:
 
 static const stress_opt_t opts[] = {
 	{ OPT_revio_bytes, "revio-bytes", TYPE_ID_UINT64_BYTES_VM, MIN_REVIO_BYTES, MAX_REVIO_BYTES, NULL },
-	{ OPT_revio_opts,  "revio-opts",  TYPE_ID_CALLBACK, 0, 0, stress_revio_opts },
+	{ OPT_revio_opts,  "revio-opts",  TYPE_ID_CALLBACK, 0, 0, (void *)stress_revio_opts },
 	END_OPT,
 };
 
