@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-madvise.h"
+#include "core-mmap.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"seal N",	"start N workers performing fcntl SEAL commands" },
@@ -58,19 +61,35 @@ static const stress_help_t help[] = {
  *  stress_seal
  *	stress file sealing
  */
-static int stress_seal(const stress_args_t *args)
+static int stress_seal(stress_args_t *args)
 {
 	int fd, ret;
 	int rc = EXIT_FAILURE;
 	const size_t page_size = args->page_size;
 	char filename[PATH_MAX];
+	char *buf;
 
+	buf = (char *)stress_mmap_populate(NULL, page_size,
+			PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (buf == MAP_FAILED) {
+		pr_inf_skip("%s: failed to allocate %zu byte buffer%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
+		return EXIT_NO_RESOURCE;
+	}
+	stress_set_vma_anon_name(buf, page_size, "write-buffer");
+	(void)shim_memset(buf, 0xff, page_size);
+	(void)stress_madvise_mergeable(buf, page_size);
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		const off_t sz = (off_t)page_size;
 		uint8_t *ptr;
-		char buf[page_size];
 		ssize_t wret;
 
 		(void)snprintf(filename, sizeof(filename), "%s-%" PRIdMAX "-%" PRIu32 "-%" PRIu32,
@@ -78,25 +97,27 @@ static int stress_seal(const stress_args_t *args)
 			args->instance, stress_mwc32());
 
 		fd = shim_memfd_create(filename, MFD_ALLOW_SEALING);
-		if (fd < 0) {
+		if (UNLIKELY(fd < 0)) {
 			if (errno == ENOSYS) {
 				pr_inf("%s: aborting, unimplemented "
 					"system call memfd_created\n", args->name);
+				(void)munmap((void *)buf, page_size);
 				return EXIT_NO_RESOURCE;
 			}
 			pr_fail("%s: memfd_create %s failed, errno=%d (%s)\n",
 				args->name, filename, errno, strerror(errno));
+			(void)munmap((void *)buf, page_size);
 			return EXIT_FAILURE;
 		}
 
-		if (ftruncate(fd, sz) < 0) {
+		if (UNLIKELY(ftruncate(fd, sz) < 0)) {
 			pr_fail("%s: ftruncate failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
 			goto err;
 		}
 
-		if (fcntl(fd, F_GET_SEALS) < 0) {
+		if (UNLIKELY(fcntl(fd, F_GET_SEALS) < 0)) {
 			pr_fail("%s: fcntl F_GET_SEALS failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -106,14 +127,14 @@ static int stress_seal(const stress_args_t *args)
 		/*
 		 *  Add shrink SEAL, file cannot be make smaller
 		 */
-		if (fcntl(fd, F_ADD_SEALS, F_SEAL_SHRINK) < 0) {
+		if (UNLIKELY(fcntl(fd, F_ADD_SEALS, F_SEAL_SHRINK) < 0)) {
 			pr_fail("%s: fcntl F_ADD_SEALS F_SEAL_SHRINK failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
 			goto err;
 		}
 		ret = ftruncate(fd, 0);
-		if ((ret == 0) || ((ret < 0) && (errno != EPERM))) {
+		if (UNLIKELY((ret == 0) || ((ret < 0) && (errno != EPERM)))) {
 			pr_fail("%s: ftruncate did not fail with EPERM as expected, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -123,14 +144,14 @@ static int stress_seal(const stress_args_t *args)
 		/*
 		 *  Add grow SEAL, file cannot be made larger
 		 */
-		if (fcntl(fd, F_ADD_SEALS, F_SEAL_GROW) < 0) {
+		if (UNLIKELY(fcntl(fd, F_ADD_SEALS, F_SEAL_GROW) < 0)) {
 			pr_fail("%s: fcntl F_ADD_SEALS F_SEAL_GROW failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
 			goto err;
 		}
 		ret = ftruncate(fd, sz + 1);
-		if ((ret == 0) || ((ret < 0) && (errno != EPERM))) {
+		if (UNLIKELY((ret == 0) || ((ret < 0) && (errno != EPERM)))) {
 			pr_fail("%s: ftruncate did not fail with EPERM as expected, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -141,32 +162,33 @@ static int stress_seal(const stress_args_t *args)
 		 *  mmap file, sealing it will return EBUSY until
 		 *  the mapping is removed
 		 */
-		ptr = mmap(NULL, (size_t)sz, PROT_WRITE, MAP_SHARED,
+		ptr = (uint8_t *)mmap(NULL, (size_t)sz, PROT_WRITE, MAP_SHARED,
 			fd, 0);
-		if (ptr == MAP_FAILED) {
+		if (UNLIKELY(ptr == MAP_FAILED)) {
 			if (errno == ENOMEM)
 				goto next;
-			pr_fail("%s: mmap failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_fail("%s: mmap of %jd bytes failed%s, errno=%d (%s)\n",
+				args->name, (intmax_t)sz,
+				stress_get_memfree_str(), errno, strerror(errno));
 			(void)close(fd);
 			goto err;
 		}
-		(void)memset(ptr, 0xea, page_size);
+		(void)shim_memset(ptr, 0xea, page_size);
 		ret = fcntl(fd, F_ADD_SEALS, F_SEAL_WRITE);
-		if ((ret == 0) || ((ret < 0) && (errno != EBUSY))) {
+		if (UNLIKELY((ret == 0) || ((ret < 0) && (errno != EBUSY)))) {
 			pr_fail("%s: fcntl F_ADD_SEALS F_SEAL_WRITE did not fail with EBUSY as expected, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			(void)munmap(ptr, (size_t)sz);
+			(void)stress_munmap_force(ptr, (size_t)sz);
 			(void)close(fd);
 			goto err;
 		}
 		(void)shim_msync(ptr, page_size, MS_SYNC);
-		(void)munmap(ptr, (size_t)sz);
+		(void)stress_munmap_force(ptr, (size_t)sz);
 
 		/*
 		 *  Now write seal the file, no more writes allowed
 		 */
-		if (fcntl(fd, F_ADD_SEALS, F_SEAL_WRITE) < 0) {
+		if (UNLIKELY(fcntl(fd, F_ADD_SEALS, F_SEAL_WRITE) < 0)) {
 			if (errno == EBUSY)
 				goto next;
 			pr_fail("%s: fcntl F_ADD_SEALS F_SEAL_WRITE failed, errno=%d (%s)\n",
@@ -174,9 +196,8 @@ static int stress_seal(const stress_args_t *args)
 			(void)close(fd);
 			goto err;
 		}
-		(void)memset(buf, 0xff, sizeof(buf));
-		wret = write(fd, buf, sizeof(buf));
-		if ((wret == 0) || ((wret < 0) && (errno != EPERM))) {
+		wret = write(fd, buf, page_size);
+		if (UNLIKELY((wret == 0) || ((wret < 0) && (errno != EPERM)))) {
 			pr_fail("%s: write on sealed file did not fail with EPERM as expected, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -190,26 +211,29 @@ static int stress_seal(const stress_args_t *args)
 next:
 		(void)close(fd);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 err:
+	(void)munmap((void *)buf, page_size);
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return rc;
 }
 
-stressor_info_t stress_seal_info = {
+const stressor_info_t stress_seal_info = {
 	.stressor = stress_seal,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_seal_info = {
-	.stressor = stress_not_implemented,
-	.class = CLASS_OS,
-	.help = help
+const stressor_info_t stress_seal_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without Linux memfd_create() system call support"
 };
 #endif
