@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +40,18 @@
 #include <sys/auxv.h>
 #endif
 
+#define BAD_ADDR	((void *)(0x10))
+#define ADDR_PAGE_MASK(virt_addr, page_size)	\
+	(volatile uint8_t *)((uintptr_t)(virt_addr) & ~(page_size - 1))
+
+static const stress_help_t help[] = {
+	{ NULL,	"sigsegv N",	 "start N workers generating segmentation faults" },
+	{ NULL,	"sigsegv-ops N", "stop after N bogo segmentation faults" },
+	{ NULL,	NULL,		 NULL }
+};
+
+#if defined(HAVE_SIGLONGJMP)
+
 static sigjmp_buf jmp_env;
 #if defined(SA_SIGINFO)
 static volatile uint8_t *fault_addr;
@@ -48,13 +60,14 @@ static volatile int signo;
 static volatile int code;
 #endif
 
-#define BAD_ADDR	((void *)(0x10))
-
-static const stress_help_t help[] = {
-	{ NULL,	"sigsegv N",	 "start N workers generating segmentation faults" },
-	{ NULL,	"sigsegv-ops N", "stop after N bogo segmentation faults" },
-	{ NULL,	NULL,		 NULL }
-};
+#if defined(__FreeBSD__) ||	\
+    defined(__NetNBSD__) ||	\
+    defined(__OpenBSD__) ||	\
+    defined(__sun__)
+#define MAX_MASK_SHIFT	(46)
+#else
+#define MAX_MASK_SHIFT	(50)
+#endif
 
 /*
  *  stress_segvhandler()
@@ -74,6 +87,7 @@ static void NORETURN MLOCKED_TEXT stress_segvhandler(
 	code = info->si_code;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #else
 static void NORETURN MLOCKED_TEXT stress_segvhandler(int signum)
@@ -81,6 +95,7 @@ static void NORETURN MLOCKED_TEXT stress_segvhandler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #endif
 
@@ -237,7 +252,7 @@ static void stress_sigsegv_vdso(void)
      defined(STRESS_ARCH_RISCV) ||	\
      defined(STRESS_ARCH_S390) ||	\
      defined(STRESS_ARCH_X86))
-	(void)clock_gettime(CLOCK_REALTIME, BAD_ADDR);
+	(void)clock_gettime(CLOCK_REALTIME, (struct timespec *)BAD_ADDR);
 #endif
 #if defined(STRESS_ARCH_ARM) ||		\
     defined(STRESS_ARCH_MIPS) ||	\
@@ -246,7 +261,7 @@ static void stress_sigsegv_vdso(void)
     defined(STRESS_ARCH_RISCV) ||	\
     defined(STRESS_ARCH_S390) ||	\
     defined(STRESS_ARCH_X86)
-	(void)gettimeofday(BAD_ADDR, NULL);
+	(void)gettimeofday((struct timeval *)BAD_ADDR, NULL);
 #endif
 }
 #endif
@@ -282,24 +297,26 @@ static int stress_sigsegv(stress_args_t *args)
 	ro_ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_READ,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (ro_ptr == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of read only page failed, "
+		pr_inf_skip("%s: failed to mmap %zu byte read only page%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, args->page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(ro_ptr, args->page_size, "ro-page");
 
-	/* Allocate write only page */
+	/* Allocate PROT_NONE page */
 	none_ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_NONE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (none_ptr == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of write only page failed, "
+		pr_inf_skip("%s: failed to mmap %zu byte PROT_NONE page%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, args->page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)ro_ptr, args->page_size);
 		return EXIT_NO_RESOURCE;
 	}
-	stress_set_vma_anon_name(ro_ptr, args->page_size, "no-page");
+	stress_set_vma_anon_name(none_ptr, args->page_size, "no-page");
 
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_GUARD_INSTALL)
@@ -307,9 +324,10 @@ static int stress_sigsegv(stress_args_t *args)
 	guard_ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_READ | PROT_WRITE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (guard_ptr == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of guard page failed, "
+		pr_inf_skip("%s: failed to mmap %zu byte guard page%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, args->page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)none_ptr, args->page_size);
 		(void)munmap((void *)ro_ptr, args->page_size);
 		return EXIT_NO_RESOURCE;
@@ -379,11 +397,20 @@ static int stress_sigsegv(stress_args_t *args)
 		if (ret) {
 			/* Signal was tripped */
 #if defined(SA_SIGINFO)
-			if (UNLIKELY(verify && expected_addr && fault_addr &&
-				     (fault_addr < expected_addr) &&
-				     (fault_addr > (expected_addr + 8)))) {
+#if defined(STRESS_ARCH_S390)
+			/*
+			 *  Some architectures only provide faulting
+			 *  address info aligned to nearest page boundary.
+			 */
+			expected_addr = ADDR_PAGE_MASK(expected_addr, args->page_size);
+			fault_addr = ADDR_PAGE_MASK(fault_addr, args->page_size);
+#endif
+			if (UNLIKELY(verify && (code != 128) && expected_addr && fault_addr &&
+				     ((fault_addr < expected_addr) ||
+				      (fault_addr > (expected_addr + 8))))) {
 				pr_fail("%s: expecting fault address %p, got %p instead\n",
-					args->name, (volatile void *)expected_addr, fault_addr);
+					args->name, (volatile void *)expected_addr,
+					(volatile void *)fault_addr);
 			}
 			if (UNLIKELY((verify &&
 				     (signo != -1) &&
@@ -406,13 +433,13 @@ static int stress_sigsegv(stress_args_t *args)
 #endif
 			stress_bogo_inc(args);
 		} else {
+retry:
 #if defined(SA_SIGINFO)
 			signo = -1;
 			code = -1;
 			fault_addr = NULL;
 			expected_addr = NULL;
 #endif
-retry:
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			switch (stress_mwc8modn(11)) {
@@ -462,6 +489,12 @@ retry:
 				stress_cpu_data_cache_flush((char *)&expected_addr, (int)sizeof(*expected_addr));
 #endif
 				stress_sigsegv_vdso();
+				/*
+				 *  calls to clock_gettime() and gettimeofday()
+				 *  may fail with -EFAULT rather than generating SIGSEGV
+				 *  (for example when the system calls are not via the
+				 *  VDSO), so we end up in the retry path
+				 */
 				goto retry;
 #endif
 			case 7:
@@ -473,7 +506,7 @@ retry:
 				*ro_ptr = 0;
 				goto retry;
 			case 8:
-				/* Read from write-only address */
+				/* Read from PROT_NONE unreadable address */
 #if defined(SA_SIGINFO)
 				expected_addr = (uint8_t *)none_ptr;
 #endif
@@ -510,9 +543,21 @@ retry:
 					mask = mask_bit;
 				}
 				mask_shift++;
+				/*
+				 *   too high and some 64 bit kernels [1] can't
+				 *   fill in si_addr correctly, so
+				 *   limit this.
+				 *
+				 *   [1] ARM64 linux, OpenIndiana x86, etc
+				 */
+				if (mask_shift > MAX_MASK_SHIFT)
+					mask_shift = 0;
 				masked_ptr &= mask;
 				masked_ptr |= mask_bit;	/* ensure top bit always set */
 				last_mask = mask;
+#if defined(SA_SIGINFO)
+				expected_addr = (uint8_t *)masked_ptr;
+#endif
 				stress_uint8_put(*(volatile uint8_t *)masked_ptr);
 				goto retry;
 			default:
@@ -539,9 +584,20 @@ tidy:
 
 const stressor_info_t stress_sigsegv_info = {
 	.stressor = stress_sigsegv,
-	.class = CLASS_SIGNAL | CLASS_OS,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
 #if defined(SA_SIGINFO)
 	.verify = VERIFY_OPTIONAL,
 #endif
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_sigsegv_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
