@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -907,13 +907,15 @@ static inline void *stress_stream_mmap(
 		if (stream_mlock)
 			(void)shim_mlock(ptr, (size_t)sz);
 #if defined(HAVE_MADVISE)
-		size_t stream_madvise;
-		int advice = MADV_NORMAL;
+		{
+			size_t stream_madvise;
+			int advice = MADV_NORMAL;
 
-		if (stress_get_setting("stream-madvise", &stream_madvise))
-			advice = stream_madvise_info[stream_madvise].advice;
+			if (stress_get_setting("stream-madvise", &stream_madvise))
+				advice = stream_madvise_info[stream_madvise].advice;
 
-		VOID_RET(int, madvise(ptr, (size_t)sz, advice));
+			VOID_RET(int, madvise(ptr, (size_t)sz, advice));
+		}
 #else
 		UNEXPECTED
 #endif
@@ -940,9 +942,6 @@ static inline uint64_t get_stream_L3_size(stress_args_t *args)
 		goto report_size;
 	}
 	max_cache_level = stress_cpu_cache_get_max_level(cpu_caches);
-	if ((max_cache_level > 0) && (max_cache_level < 3) && (!args->instance))
-		pr_inf("%s: no L3 cache, using L%" PRIu16 " size instead\n",
-			args->name, max_cache_level);
 
 	cache = stress_cpu_cache_get(cpu_caches, max_cache_level);
 	if (!cache) {
@@ -959,6 +958,17 @@ static inline uint64_t get_stream_L3_size(stress_args_t *args)
 		stress_free_cpu_caches(cpu_caches);
 		goto report_size;
 	}
+	if ((max_cache_level > 0) && (max_cache_level < 3) && (!args->instance)) {
+		if (cache->size > cache_size) {
+			pr_inf("%s: no L3 cache, using L%" PRIu16 " size instead\n",
+				args->name, max_cache_level);
+		} else {
+			pr_inf("%s: no L3 cache, and L%" PRIu16 " size is small, using built-in default instead\n",
+				args->name, max_cache_level);
+			goto report_size;
+
+		}
+	}
 	cache_size = cache->size;
 
 	stress_free_cpu_caches(cpu_caches);
@@ -1153,8 +1163,8 @@ static int OPTIMIZE3 TARGET_CLONES stress_stream_verify(
 static int stress_stream(stress_args_t *args)
 {
 	int rc = EXIT_FAILURE;
-	double *a = MAP_FAILED, *b = MAP_FAILED, *c = MAP_FAILED;
-	size_t *idx1 = MAP_FAILED, *idx2 = MAP_FAILED, *idx3 = MAP_FAILED;
+	double *a = (double *)MAP_FAILED, *b = (double *)MAP_FAILED, *c = (double *)MAP_FAILED;
+	size_t *idx1 = (size_t *)MAP_FAILED, *idx2 = (size_t *)MAP_FAILED, *idx3 = (size_t *)MAP_FAILED;
 	const double q = 3.0;
 	double old_checksum = -1.0;
 	double fp_ops = 0.0, dt;
@@ -1199,18 +1209,18 @@ static int stress_stream(stress_args_t *args)
 	}
 
 	if (stress_instance_zero(args)) {
-		pr_inf("%s: stressor loosely based on a variant of the "
-			"STREAM benchmark code\n", args->name);
-		pr_inf("%s: do NOT submit any of these results "
-			"to the STREAM benchmark results\n", args->name);
 		if (guess) {
 			pr_inf("%s: cannot determine CPU L3 cache size, "
 				"defaulting to %" PRIu64 "K\n",
 				args->name, L3 / 1024);
 		} else {
-			pr_inf("%s: Using cache size of %" PRIu64 "K\n",
+			pr_inf("%s: using cache size of %" PRIu64 "K\n",
 				args->name, L3 / 1024);
 		}
+		pr_inf("%s: stressor loosely based on a variant of the "
+			"STREAM benchmark code\n", args->name);
+		pr_inf("%s: do NOT submit any of these results "
+			"to the STREAM benchmark results\n", args->name);
 	}
 
 	/* ..and shared amongst all the STREAM stressor instances */
@@ -1218,12 +1228,7 @@ static int stress_stream(stress_args_t *args)
 	if (L3 < args->page_size)
 		L3 = args->page_size;
 
-	/*
-	 *  Each array must be at least 4 x the
-	 *  size of the L3 cache
-	 */
-	sz = (L3 * 4);
-	n = sz / sizeof(*a);
+	n = L3 / sizeof(*a);
 	/*
 	 *  n must be a multiple of the max unroll size (8)
 	 */
@@ -1231,33 +1236,38 @@ static int stress_stream(stress_args_t *args)
 	sz = n * sizeof(*a);
 	sz_idx = n * sizeof(size_t);
 
-	a = stress_stream_mmap(args, sz, stream_mlock);
+	if (stress_instance_zero(args)) {
+		pr_inf("%s: using 3 cache sized buffers for stream operations, total %" PRIu64 "K per stream instance\n",
+			args->name, 3 * (sz / 1024));
+	}
+
+	a = (double *)stress_stream_mmap(args, sz, stream_mlock);
 	if (a == MAP_FAILED)
 		goto err_unmap;
-	b = stress_stream_mmap(args, sz, stream_mlock);
+	b = (double *)stress_stream_mmap(args, sz, stream_mlock);
 	if (b == MAP_FAILED)
 		goto err_unmap;
-	c = stress_stream_mmap(args, sz, stream_mlock);
+	c = (double *)stress_stream_mmap(args, sz, stream_mlock);
 	if (c == MAP_FAILED)
 		goto err_unmap;
 
 	switch (stream_index) {
 	case 3:
-		idx3 = stress_stream_mmap(args, sz_idx, stream_mlock);
+		idx3 = (size_t *)stress_stream_mmap(args, sz_idx, stream_mlock);
 		if (idx3 == MAP_FAILED)
 			goto err_unmap;
 		stress_stream_init_index(idx3, n);
 		goto case_stream_index_2;
 	case 2:
 case_stream_index_2:
-		idx2 = stress_stream_mmap(args, sz_idx, stream_mlock);
+		idx2 = (size_t *)stress_stream_mmap(args, sz_idx, stream_mlock);
 		if (idx2 == MAP_FAILED)
 			goto err_unmap;
 		stress_stream_init_index(idx2, n);
 		goto case_stream_index_1;
 	case 1:
 case_stream_index_1:
-		idx1 = stress_stream_mmap(args, sz_idx, stream_mlock);
+		idx1 = (size_t *)stress_stream_mmap(args, sz_idx, stream_mlock);
 		if (idx1 == MAP_FAILED)
 			goto err_unmap;
 		stress_stream_init_index(idx1, n);
@@ -1342,7 +1352,7 @@ static const char *stress_stream_madvise(const size_t i)
 static const stress_opt_t opts[] = {
 	{ OPT_stream_index,    "stream-index",    TYPE_ID_UINT32, 0, 3, NULL },
 	{ OPT_stream_l3_size,  "stream-l3-size",  TYPE_ID_UINT64_BYTES_VM, MIN_STREAM_L3_SIZE, MAX_STREAM_L3_SIZE, NULL },
-	{ OPT_stream_madvise,  "stream-madvise",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_stream_madvise },
+	{ OPT_stream_madvise,  "stream-madvise",  TYPE_ID_SIZE_T_METHOD, 0, 0, (void *)stress_stream_madvise },
 	{ OPT_stream_mlock,    "stream-mlock",    TYPE_ID_BOOL, 0, 1, NULL },
 	{ OPT_stream_prefetch, "stream-prefetch", TYPE_ID_BOOL, 0, 1, NULL },
 	END_OPT,
