@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -64,7 +64,7 @@ static int stress_sync_allocate(
 
 	ret = ftruncate(fd, 0);
 	if (UNLIKELY(ret < 0)) {
-		pr_err("%s: ftruncate failed: errno=%d (%s)%s\n",
+		pr_err("%s: ftruncate failed, errno=%d (%s)%s\n",
 			args->name, errno, strerror(errno), fs_type);
 		return -errno;
 	}
@@ -74,7 +74,7 @@ static int stress_sync_allocate(
 	if (UNLIKELY(ret < 0)) {
 		if ((errno == ENOSPC) || (errno == EINTR))
 			return -errno;
-		pr_fail("%s: fdatasync failed: errno=%d (%s)%s\n",
+		pr_fail("%s: fdatasync failed, errno=%d (%s)%s\n",
 			args->name, errno, strerror(errno), fs_type);
 		return -errno;
 	}
@@ -88,7 +88,7 @@ static int stress_sync_allocate(
 			return 0;
 		if (errno == ENOSPC)
 			return -errno;
-		pr_err("%s: fallocate failed: errno=%d (%s)%s\n",
+		pr_err("%s: fallocate failed, errno=%d (%s)%s\n",
 			args->name, errno, strerror(errno), fs_type);
 		return -errno;
 	}
@@ -103,19 +103,23 @@ static int stress_sync_file(stress_args_t *args)
 {
 	int fd, ret, rc = EXIT_SUCCESS;
 	const int bad_fd = stress_get_bad_fd();
-	off_t sync_file_bytes = DEFAULT_SYNC_FILE_BYTES;
+	off_t sync_file_bytes, sync_file_bytes_total = DEFAULT_SYNC_FILE_BYTES;
 	char filename[PATH_MAX];
 	const char *fs_type;
 
-	if (!stress_get_setting("sync_file-bytes", &sync_file_bytes)) {
+	if (!stress_get_setting("sync-file-bytes", &sync_file_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			sync_file_bytes = MAXIMIZED_FILE_SIZE;
+			sync_file_bytes_total = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			sync_file_bytes = MIN_SYNC_FILE_BYTES;
+			sync_file_bytes_total = MIN_SYNC_FILE_BYTES;
 	}
-	sync_file_bytes /= args->instances;
-	if (sync_file_bytes < (off_t)MIN_SYNC_FILE_BYTES)
+	sync_file_bytes = sync_file_bytes_total / args->instances;
+	if (sync_file_bytes < (off_t)MIN_SYNC_FILE_BYTES) {
 		sync_file_bytes = (off_t)MIN_SYNC_FILE_BYTES;
+		sync_file_bytes_total = sync_file_bytes * args->instances;
+	}
+	if (stress_instance_zero(args))
+		stress_fs_usage_bytes(args, sync_file_bytes, sync_file_bytes_total);
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
@@ -125,7 +129,7 @@ static int stress_sync_file(stress_args_t *args)
 		filename, sizeof(filename), stress_mwc32());
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		if ((errno == ENFILE) || (errno == ENOMEM) || (errno == ENOSPC)) {
-			pr_inf_skip("%s: cannot create file to sync on, skipping stressor: errno=%d (%s)\n",
+			pr_inf_skip("%s: cannot create file to sync on, skipping stressor, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
 		}
@@ -197,7 +201,7 @@ static int stress_sync_file(stress_args_t *args)
 		VOID_RET(int, shim_sync_file_range(fd, 0, -1, mode));
 
 		/* Sync from halfway along file to end */
-		VOID_RET(int, shim_sync_file_range(fd, sync_file_bytes << 2, 0, mode));
+		VOID_RET(int, shim_sync_file_range(fd, sync_file_bytes >> 1, 0, mode));
 
 		ret = stress_sync_allocate(args, fd, fs_type, sync_file_bytes);
 		if (UNLIKELY(ret < 0)) {
@@ -262,7 +266,7 @@ err:
 
 const stressor_info_t stress_sync_file_info = {
 	.stressor = stress_sync_file,
-	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -270,7 +274,7 @@ const stressor_info_t stress_sync_file_info = {
 #else
 const stressor_info_t stress_sync_file_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
