@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -71,6 +71,8 @@ again:
 			args->name, errno, strerror(errno));
 		return -1;
 	} else if (pid == 0) {
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
+		stress_set_make_it_fail();
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -110,8 +112,9 @@ static void stress_tee_pipe_write(stress_args_t *args, int fds[2])
 					args->name, errno, strerror(errno));
 				break;
 			}
+		} else {
+			data.counter++;
 		}
-		data.counter++;
 	}
 	(void)close(fds[1]);
 }
@@ -139,6 +142,7 @@ static void stress_tee_pipe_read(stress_args_t *args, int fds[2])
 			if (UNLIKELY(ret < 0)) {
 				switch (errno) {
 				case EPIPE:
+					(void)close(fds[0]);
 					return;
 				case EAGAIN:
 				case EINTR:
@@ -146,6 +150,7 @@ static void stress_tee_pipe_read(stress_args_t *args, int fds[2])
 				default:
 					pr_fail("%s: unexpected read error, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					(void)close(fds[0]);
 					return;
 				}
 			} else {
@@ -291,6 +296,8 @@ static int stress_tee(stress_args_t *args)
 					args->name);
 				ret = EXIT_NO_RESOURCE;
 				goto tidy_child2;
+			default:
+				break;
 			}
 			pr_fail("%s: tee failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -301,9 +308,9 @@ do_splice:
 		while (len > 0) {
 			slen = splice(pipe_in[0], NULL, fd, NULL,
 				(size_t)len, SPLICE_F_MOVE);
-			if (UNLIKELY(errno == EINTR))
-				break;
 			if (UNLIKELY(slen < 0)) {
+				if (UNLIKELY(errno == EINTR))
+					break;
 				pr_err("%s: splice failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				goto tidy_child2;
@@ -340,14 +347,14 @@ tidy_fd:
 
 const stressor_info_t stress_tee_info = {
 	.stressor = stress_tee,
-	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
+	.classifier = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_tee_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
+	.classifier = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without tee() system call or undefined SPLICE_F_NONBLOCK"
