@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2025 Colin Ian King.
+ * Copyright (C) 2021-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -86,6 +86,7 @@ static void stress_sigill_handler(int signum)
 	(void)signum;
 
 	siglongjmp(jmpbuf, 1);
+	stress_no_return();
 }
 
 /*
@@ -162,7 +163,7 @@ static inline uint64_t rdtsc(void)
 	return stress_asm_x86_rdtsc();
 }
 
-#elif defined(STRESS_ARCH_PPC64) &&		\
+#elif (defined(STRESS_ARCH_PPC64) || defined(STRESS_ARCH_PPC)) &&	\
       defined(HAVE_SYS_PLATFORM_PPC_H) &&	\
       defined(HAVE_PPC_GET_TIMEBASE)
 
@@ -243,7 +244,7 @@ static void stress_tsc_check(
 	if (((old_tsc ^ tsc) >> 63) == 1)
 		return;
 
-	pr_fail("%s: TSC not monitonically increasing, TSC %" PRIx64 " vs previous TSC %" PRIx64 "\n",
+	pr_fail("%s: TSC not monotonically increasing, TSC %" PRIx64 " vs previous TSC %" PRIx64 "\n",
 		args->name, tsc, old_tsc);
 	*ret = EXIT_FAILURE;
 	return;
@@ -294,7 +295,7 @@ do {			\
 } while (0)
 
 /*
- *  Unrolled 32 times, verify monitonically increasing at end
+ *  Unrolled 32 times, verify monotonically increasing at end
  */
 #define TSCx32_verify(args, tsc, old_tsc, ret)	\
 do {			\
@@ -382,7 +383,7 @@ do {					\
 } while (0)
 
 /*
- *  Unrolled 32 times, verify monitonically increasing at end
+ *  Unrolled 32 times, verify monotonically increasing at end
  */
 #define TSCPx32_verify(args, tsc, old_tsc, ret)	\
 do {					\
@@ -432,7 +433,7 @@ do {					\
 	do {			\
 		rdtsc();	\
 		lfence();	\
-	} while (0);
+	} while (0)
 
 /*
  *  Unrolled 32 times, no verify with lfence
@@ -477,7 +478,7 @@ do {			\
 } while (0)
 
 /*
- *  Unrolled 32 times, verify monitonically increasing at end, with lfence
+ *  Unrolled 32 times, verify monotonically increasing at end, with lfence
  */
 #define TSCx32_lfence_verify(args, tsc, old_tsc, ret)	\
 do {			\
@@ -690,7 +691,7 @@ static int stress_tsc(stress_args_t *args)
 		const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 		double duration = 0.0, count;
 
-		tsc_func(args, verify, &duration);
+		ret = tsc_func(args, verify, &duration);
 		count = 32.0 * 4.0 * (double)stress_bogo_get(args);
 		duration = (count > 0.0) ? duration / count : 0.0;
 		stress_metrics_set(args, 0, "nanosecs per time counter read",
@@ -704,7 +705,7 @@ static int stress_tsc(stress_args_t *args)
 const stressor_info_t stress_tsc_info = {
 	.stressor = stress_tsc,
 	.supported = stress_tsc_supported,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.verify = VERIFY_OPTIONAL,
 	.opts = opts,
 	.help = help
@@ -721,7 +722,7 @@ static int stress_tsc_supported(const char *name)
 const stressor_info_t stress_tsc_info = {
 	.stressor = stress_unimplemented,
 	.supported = stress_tsc_supported,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.verify = VERIFY_OPTIONAL,
 	.opts = opts,
 	.help = help,
